@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
 #
-# Copyright 2012-2020 BigML
+# Copyright 2012-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -40,45 +39,50 @@ import json
 
 from bigml.bigmlconnection import BigMLConnection
 from bigml.domain import BIGML_PROTOCOL
-from bigml.resourcehandler import ResourceHandler, check_resource
-from bigml.sourcehandler import SourceHandler
-from bigml.datasethandler import DatasetHandler
-from bigml.modelhandler import ModelHandler
-from bigml.ensemblehandler import EnsembleHandler
-from bigml.predictionhandler import PredictionHandler
-from bigml.clusterhandler import ClusterHandler
-from bigml.centroidhandler import CentroidHandler
-from bigml.anomalyhandler import AnomalyHandler
-from bigml.anomalyscorehandler import AnomalyScoreHandler
-from bigml.evaluationhandler import EvaluationHandler
-from bigml.batchpredictionhandler import BatchPredictionHandler
-from bigml.batchcentroidhandler import BatchCentroidHandler
-from bigml.batchanomalyscorehandler import BatchAnomalyScoreHandler
-from bigml.projecthandler import ProjectHandler
-from bigml.samplehandler import SampleHandler
-from bigml.correlationhandler import CorrelationHandler
-from bigml.statisticaltesthandler import StatisticalTestHandler
-from bigml.logistichandler import LogisticRegressionHandler
-from bigml.associationhandler import AssociationHandler
-from bigml.associationsethandler import AssociationSetHandler
-from bigml.configurationhandler import ConfigurationHandler
-from bigml.topicmodelhandler import TopicModelHandler
-from bigml.topicdistributionhandler import TopicDistributionHandler
-from bigml.batchtopicdistributionhandler import BatchTopicDistributionHandler
-from bigml.timeserieshandler import TimeSeriesHandler
-from bigml.forecasthandler import ForecastHandler
-from bigml.deepnethandler import DeepnetHandler
-from bigml.optimlhandler import OptimlHandler
-from bigml.fusionhandler import FusionHandler
-from bigml.pcahandler import PCAHandler
-from bigml.projectionhandler import ProjectionHandler
-from bigml.linearhandler import LinearRegressionHandler
-from bigml.batchprojectionhandler import BatchProjectionHandler
-from bigml.scripthandler import ScriptHandler
-from bigml.executionhandler import ExecutionHandler
-from bigml.libraryhandler import LibraryHandler
 from bigml.constants import STORAGE, ALL_FIELDS
-from bigml.externalconnectorhandler import ExternalConnectorHandler
+from bigml.util import is_in_progress
+from bigml.api_handlers.resourcehandler import ResourceHandlerMixin
+from bigml.api_handlers.sourcehandler import SourceHandlerMixin
+from bigml.api_handlers.datasethandler import DatasetHandlerMixin
+from bigml.api_handlers.modelhandler import ModelHandlerMixin
+from bigml.api_handlers.ensemblehandler import EnsembleHandlerMixin
+from bigml.api_handlers.predictionhandler import PredictionHandlerMixin
+from bigml.api_handlers.clusterhandler import ClusterHandlerMixin
+from bigml.api_handlers.centroidhandler import CentroidHandlerMixin
+from bigml.api_handlers.anomalyhandler import AnomalyHandlerMixin
+from bigml.api_handlers.anomalyscorehandler import AnomalyScoreHandlerMixin
+from bigml.api_handlers.evaluationhandler import EvaluationHandlerMixin
+from bigml.api_handlers.batchpredictionhandler import BatchPredictionHandlerMixin
+from bigml.api_handlers.batchcentroidhandler import BatchCentroidHandlerMixin
+from bigml.api_handlers.batchanomalyscorehandler \
+    import BatchAnomalyScoreHandlerMixin
+from bigml.api_handlers.projecthandler import ProjectHandlerMixin
+from bigml.api_handlers.samplehandler import SampleHandlerMixin
+from bigml.api_handlers.correlationhandler import CorrelationHandlerMixin
+from bigml.api_handlers.statisticaltesthandler import StatisticalTestHandlerMixin
+from bigml.api_handlers.logistichandler import LogisticRegressionHandlerMixin
+from bigml.api_handlers.associationhandler import AssociationHandlerMixin
+from bigml.api_handlers.associationsethandler import AssociationSetHandlerMixin
+from bigml.api_handlers.configurationhandler import ConfigurationHandlerMixin
+from bigml.api_handlers.topicmodelhandler import TopicModelHandlerMixin
+from bigml.api_handlers.topicdistributionhandler \
+    import TopicDistributionHandlerMixin
+from bigml.api_handlers.batchtopicdistributionhandler  \
+    import BatchTopicDistributionHandlerMixin
+from bigml.api_handlers.timeserieshandler import TimeSeriesHandlerMixin
+from bigml.api_handlers.forecasthandler import ForecastHandlerMixin
+from bigml.api_handlers.deepnethandler import DeepnetHandlerMixin
+from bigml.api_handlers.optimlhandler import OptimlHandlerMixin
+from bigml.api_handlers.fusionhandler import FusionHandlerMixin
+from bigml.api_handlers.pcahandler import PCAHandlerMixin
+from bigml.api_handlers.projectionhandler import ProjectionHandlerMixin
+from bigml.api_handlers.linearhandler import LinearRegressionHandlerMixin
+from bigml.api_handlers.batchprojectionhandler import BatchProjectionHandlerMixin
+from bigml.api_handlers.scripthandler import ScriptHandlerMixin
+from bigml.api_handlers.executionhandler import ExecutionHandlerMixin
+from bigml.api_handlers.libraryhandler import LibraryHandlerMixin
+from bigml.api_handlers.externalconnectorhandler import \
+    ExternalConnectorHandlerMixin
 
 
 # Repeating constants and functions for backwards compatibility
@@ -116,7 +120,7 @@ from bigml.constants import (
     IRREGULAR_PLURALS, RESOURCES_WITH_FIELDS, FIELDS_PARENT,
     EXTERNAL_CONNECTOR_PATH, EXTERNAL_CONNECTOR_RE)
 
-from bigml.resourcehandler import (
+from bigml.api_handlers.resourcehandler import (
     get_resource, get_resource_type, check_resource_type, get_source_id,
     get_dataset_id, get_model_id, get_ensemble_id, get_evaluation_id,
     get_cluster_id, get_centroid_id, get_anomaly_id, get_anomaly_score_id,
@@ -193,8 +197,21 @@ def count(listing):
     """
     if 'meta' in listing and 'query_total' in listing['meta']:
         return listing['meta']['query_total']
+    return None
 
 
+def filter_kwargs(kwargs, list_of_keys, out=False):
+    """Creates a new dict with the selected list of keys if present
+    If `out` is set to True, the keys in the list are removed
+    If `out` is set to False, only the keys in the list are kept
+
+    """
+    new_kwargs = {}
+    for key in kwargs:
+        if (key not in list_of_keys and out) or \
+               (key in list_of_keys and not out):
+            new_kwargs[key] = kwargs[key]
+    return new_kwargs
 
 def get_fields(resource):
     """Returns the field information in a resource dictionary structure
@@ -209,34 +226,34 @@ def get_fields(resource):
     if resource_type in RESOURCES_WITH_FIELDS:
         resource = resource.get('object', resource)
         # fields structure
-        if resource_type in FIELDS_PARENT.keys():
+        if resource_type in list(FIELDS_PARENT.keys()):
             fields = resource[FIELDS_PARENT[resource_type]].get('fields', {})
         else:
             fields = resource.get('fields', {})
 
         if resource_type == SAMPLE_PATH:
-            fields = dict([(field['id'], field) for field in
-                           fields])
+            fields = {field['id']: field for field in fields}
     return fields
 
 
-class BigML(ExternalConnectorHandler,
-            LinearRegressionHandler, BatchProjectionHandler,
-            ProjectionHandler, PCAHandler,
-            ConfigurationHandler, FusionHandler,
-            OptimlHandler,
-            DeepnetHandler, ForecastHandler, TimeSeriesHandler,
-            BatchTopicDistributionHandler, TopicDistributionHandler,
-            TopicModelHandler, LibraryHandler, ExecutionHandler, ScriptHandler,
-            AssociationSetHandler, AssociationHandler,
-            LogisticRegressionHandler,
-            StatisticalTestHandler, CorrelationHandler,
-            SampleHandler, ProjectHandler,
-            BatchAnomalyScoreHandler, BatchCentroidHandler,
-            BatchPredictionHandler, EvaluationHandler, AnomalyScoreHandler,
-            AnomalyHandler, CentroidHandler, ClusterHandler, PredictionHandler,
-            EnsembleHandler, ModelHandler, DatasetHandler,
-            SourceHandler, ResourceHandler, BigMLConnection):
+class BigML(ExternalConnectorHandlerMixin,
+            LinearRegressionHandlerMixin, BatchProjectionHandlerMixin,
+            ProjectionHandlerMixin, PCAHandlerMixin,
+            ConfigurationHandlerMixin, FusionHandlerMixin,
+            OptimlHandlerMixin,
+            DeepnetHandlerMixin, ForecastHandlerMixin, TimeSeriesHandlerMixin,
+            BatchTopicDistributionHandlerMixin, TopicDistributionHandlerMixin,
+            TopicModelHandlerMixin, LibraryHandlerMixin, ExecutionHandlerMixin,
+            ScriptHandlerMixin, AssociationSetHandlerMixin,
+            AssociationHandlerMixin, LogisticRegressionHandlerMixin,
+            StatisticalTestHandlerMixin, CorrelationHandlerMixin,
+            SampleHandlerMixin, ProjectHandlerMixin,
+            BatchAnomalyScoreHandlerMixin, BatchCentroidHandlerMixin,
+            BatchPredictionHandlerMixin, EvaluationHandlerMixin,
+            AnomalyScoreHandlerMixin, AnomalyHandlerMixin,
+            CentroidHandlerMixin, ClusterHandlerMixin, PredictionHandlerMixin,
+            EnsembleHandlerMixin, ModelHandlerMixin, DatasetHandlerMixin,
+            SourceHandlerMixin, ResourceHandlerMixin, BigMLConnection):
     """Entry point to create, retrieve, list, update, and delete
     BigML resources.
 
@@ -284,51 +301,50 @@ class BigML(ExternalConnectorHandler,
         given by the organization administrator.
 
         """
-
         BigMLConnection.__init__(self, username=username, api_key=api_key,
                                  debug=debug,
                                  set_locale=set_locale, storage=storage,
                                  domain=domain, project=project,
                                  organization=organization,
                                  short_debug=short_debug)
-        ResourceHandler.__init__(self)
-        SourceHandler.__init__(self)
-        DatasetHandler.__init__(self)
-        ModelHandler.__init__(self)
-        EnsembleHandler.__init__(self)
-        PredictionHandler.__init__(self)
-        ClusterHandler.__init__(self)
-        CentroidHandler.__init__(self)
-        AnomalyHandler.__init__(self)
-        AnomalyScoreHandler.__init__(self)
-        EvaluationHandler.__init__(self)
-        BatchPredictionHandler.__init__(self)
-        BatchCentroidHandler.__init__(self)
-        BatchAnomalyScoreHandler.__init__(self)
-        ProjectHandler.__init__(self)
-        SampleHandler.__init__(self)
-        CorrelationHandler.__init__(self)
-        StatisticalTestHandler.__init__(self)
-        LogisticRegressionHandler.__init__(self)
-        AssociationHandler.__init__(self)
-        AssociationSetHandler.__init__(self)
-        ScriptHandler.__init__(self)
-        ExecutionHandler.__init__(self)
-        LibraryHandler.__init__(self)
-        TopicModelHandler.__init__(self)
-        TopicDistributionHandler.__init__(self)
-        BatchTopicDistributionHandler.__init__(self)
-        TimeSeriesHandler.__init__(self)
-        ForecastHandler.__init__(self)
-        DeepnetHandler.__init__(self)
-        OptimlHandler.__init__(self)
-        FusionHandler.__init__(self)
-        ConfigurationHandler.__init__(self)
-        PCAHandler.__init__(self)
-        ProjectionHandler.__init__(self)
-        BatchProjectionHandler.__init__(self)
-        LinearRegressionHandler.__init__(self)
-        ExternalConnectorHandler.__init__(self)
+        ResourceHandlerMixin.__init__(self)
+        SourceHandlerMixin.__init__(self)
+        DatasetHandlerMixin.__init__(self)
+        ModelHandlerMixin.__init__(self)
+        EnsembleHandlerMixin.__init__(self)
+        PredictionHandlerMixin.__init__(self)
+        ClusterHandlerMixin.__init__(self)
+        CentroidHandlerMixin.__init__(self)
+        AnomalyHandlerMixin.__init__(self)
+        AnomalyScoreHandlerMixin.__init__(self)
+        EvaluationHandlerMixin.__init__(self)
+        BatchPredictionHandlerMixin.__init__(self)
+        BatchCentroidHandlerMixin.__init__(self)
+        BatchAnomalyScoreHandlerMixin.__init__(self)
+        ProjectHandlerMixin.__init__(self)
+        SampleHandlerMixin.__init__(self)
+        CorrelationHandlerMixin.__init__(self)
+        StatisticalTestHandlerMixin.__init__(self)
+        LogisticRegressionHandlerMixin.__init__(self)
+        AssociationHandlerMixin.__init__(self)
+        AssociationSetHandlerMixin.__init__(self)
+        ScriptHandlerMixin.__init__(self)
+        ExecutionHandlerMixin.__init__(self)
+        LibraryHandlerMixin.__init__(self)
+        TopicModelHandlerMixin.__init__(self)
+        TopicDistributionHandlerMixin.__init__(self)
+        BatchTopicDistributionHandlerMixin.__init__(self)
+        TimeSeriesHandlerMixin.__init__(self)
+        ForecastHandlerMixin.__init__(self)
+        DeepnetHandlerMixin.__init__(self)
+        OptimlHandlerMixin.__init__(self)
+        FusionHandlerMixin.__init__(self)
+        ConfigurationHandlerMixin.__init__(self)
+        PCAHandlerMixin.__init__(self)
+        ProjectionHandlerMixin.__init__(self)
+        BatchProjectionHandlerMixin.__init__(self)
+        LinearRegressionHandlerMixin.__init__(self)
+        ExternalConnectorHandlerMixin.__init__(self)
         self.status_url = "%s%s" % (self.url, STATUS_PATH)
 
 
@@ -336,11 +352,12 @@ class BigML(ExternalConnectorHandler,
         for resource_type in RESOURCE_RE:
             method_name = RENAMED_RESOURCES.get(resource_type, resource_type)
             self.getters[resource_type] = getattr(self, "get_%s" % method_name)
-        self.creaters = {}
+        self.creators = {}
         for resource_type in RESOURCE_RE:
             method_name = RENAMED_RESOURCES.get(resource_type, resource_type)
-            self.creaters[resource_type] = getattr(self,
+            self.creators[resource_type] = getattr(self,
                                                    "create_%s" % method_name)
+        self.creaters = self.creators # to be deprecated
         self.updaters = {}
         for resource_type in RESOURCE_RE:
             method_name = RENAMED_RESOURCES.get(resource_type, resource_type)
@@ -359,35 +376,104 @@ class BigML(ExternalConnectorHandler,
             self.listers[resource_type] = getattr(self,
                                                   "list_%s" % method_name)
 
+    def create(self, resource_type, *args, **kwargs):
+        """Create resources
+
+        """
+        finished = kwargs.get('finished', True)
+        create_kwargs = filter_kwargs(kwargs,
+                                      ['query_string', 'finished'],
+                                      out=True)
+        try:
+            resource_info = self.creators[resource_type](*args,
+                                                         **create_kwargs)
+        except KeyError:
+            raise ValueError("Failed to create %s. This kind of resource"
+                             " does not exist." % resource_type)
+        if finished and is_in_progress(resource_info):
+            ok_kwargs = filter_kwargs(kwargs, ['query_string'])
+            ok_kwargs.update({"error_retries": 5, "debug": self.debug})
+            self.ok(resource_info, **ok_kwargs)
+        return resource_info
+
+    def get(self, resource, **kwargs):
+        """Method to get resources
+
+        """
+        finished = kwargs.get('finished', True)
+        get_kwargs = filter_kwargs(kwargs,
+                                   ['finished'],
+                                   out=True)
+        try:
+            resource_type = get_resource_type(resource)
+            resource_info = self.getters[resource_type](resource, **get_kwargs)
+        except KeyError:
+            raise ValueError("%s is not a resource or ID." % resource)
+        if finished and is_in_progress(resource_info):
+            ok_kwargs = filter_kwargs(kwargs, ['query_string'])
+            ok_kwargs.update({"error_retries": 5, "debug": self.debug})
+            self.ok(resource_info, **ok_kwargs)
+        return resource_info
+
+    def update(self, resource, changes, **kwargs):
+        """Method to update resources
+
+        """
+        finished = kwargs.get('finished', True)
+        try:
+            resource_type = get_resource_type(resource)
+            update_kwargs = filter_kwargs(kwargs,
+                                          ['query_string', 'finished'],
+                                          out=True)
+            resource_info = self.updaters[resource_type](resource, changes,
+                                                         **update_kwargs)
+        except KeyError:
+            raise ValueError("%s is not a resource or ID." % resource)
+        if finished and is_in_progress(resource_info):
+            ok_kwargs = filter_kwargs(kwargs, ['query_string'])
+            ok_kwargs.update({"error_retries": 5, "debug": self.debug})
+            self.ok(resource_info, **ok_kwargs)
+        return resource_info
+
+    def delete(self, resource, **kwargs):
+        """Method to delete resources
+
+        """
+        try:
+            resource_type = get_resource_type(resource)
+            return self.deleters[resource_type](resource, **kwargs)
+        except KeyError:
+            raise ValueError("%s is not a resource." % resource)
+
     def connection_info(self):
         """Printable string: domain where the connection is bound and the
            credentials used.
 
         """
-        info = u"Connecting to:\n"
-        info += u"    %s\n" % self.general_domain
+        info = "Connecting to:\n"
+        info += "    %s\n" % self.general_domain
         if self.general_protocol != BIGML_PROTOCOL:
-            info += u"    using %s protocol\n" % self.general_protocol
-        info += u"    SSL verification %s\n" % (
+            info += "    using %s protocol\n" % self.general_protocol
+        info += "    SSL verification %s\n" % (
             "on" if self.verify else "off")
         short = "(shortened)" if self.short_debug else ""
         if self.debug:
-            info += u"    Debug on %s\n" % short
+            info += "    Debug on %s\n" % short
         if self.general_domain != self.prediction_domain:
-            info += u"    %s (predictions only)\n" % self.prediction_domain
+            info += "    %s (predictions only)\n" % self.prediction_domain
             if self.prediction_protocol != BIGML_PROTOCOL:
-                info += u"    using %s protocol\n" % self.prediction_protocol
-            info += u"    SSL verification %s\n" % (
+                info += "    using %s protocol\n" % self.prediction_protocol
+            info += "    SSL verification %s\n" % (
                 "on" if self.verify_prediction else "off")
 
         if self.project or self.organization:
-            info += u"    Scope info: %s\n" % \
-                u"%s\n                %s" % (self.organization or "",
-                                             self.project or "")
+            info += "    Scope info: %s\n" % \
+                "%s\n                %s" % (self.organization or "",
+                                            self.project or "")
 
 
-        info += u"\nAuthentication string:\n"
-        info += u"    %s\n" % self.auth[1:]
+        info += "\nAuthentication string:\n"
+        info += "    %s\n" % self.auth[1:]
         return info
 
     def get_account_status(self):
@@ -409,15 +495,26 @@ class BigML(ExternalConnectorHandler,
         """
         if self.organization is not None:
             status = self._status(self.status_url,
-                                  organization=self.organization)["object"]
+                                  organization=self.organization)
         else:
-            status = self._status(self.status_url)["object"]
+            status = self._status(self.status_url)
+        if status["error"] is None:
+            status = status.get("object", {})
+            return {
+                "tasks": status.get("tasks"),
+                "max_tasks": status.get("subscription", {}).get("max_tasks"),
+                "available_tasks": (status.get("subscription",
+                                               {}).get("max_tasks")
+                                    - status.get("tasks")),
+                "tasks_in_progress": status.get("tasks_in_progress"),
+                "error": None}
+
         return {
-            "tasks": status["tasks"],
-            "max_tasks": status["subscription"]["max_tasks"],
-            "available_tasks": (status["subscription"]["max_tasks"]
-                                - status["tasks"]),
-            "tasks_in_progress": status["tasks_in_progress"]}
+            "tasks": 0,
+            "max_tasks": 0,
+            "available_tasks": 0,
+            "tasks_in_progress": 0,
+            "error": status["error"]}
 
     def get_fields(self, resource):
         """Retrieve fields used by a resource.
@@ -429,7 +526,7 @@ class BigML(ExternalConnectorHandler,
 
         if isinstance(resource, dict) and 'resource' in resource:
             resource_id = resource['resource']
-        elif isinstance(resource, basestring) and get_resource_type(resource) \
+        elif isinstance(resource, str) and get_resource_type(resource) \
                 in RESOURCES_WITH_FIELDS:
             resource_id = resource
             resource = self.retrieve_resource(resource,
@@ -445,7 +542,6 @@ class BigML(ExternalConnectorHandler,
         except KeyError:
             resource = self._get("%s%s" % (self.url, resource_id))
             fields = get_fields(resource)
-
         return fields
 
     def pprint(self, resource, out=sys.stdout):
@@ -479,7 +575,7 @@ class BigML(ExternalConnectorHandler,
                     resource['object']['fields'][
                         resource['object']['objective_fields'][0]]['name'])
                 input_data = {}
-                for key, value in resource['object']['input_data'].items():
+                for key, value in list(resource['object']['input_data'].items()):
                     try:
                         name = resource['object']['fields'][key]['name']
                     except KeyError:
@@ -506,12 +602,12 @@ class BigML(ExternalConnectorHandler,
             status = get_status(resource)
             code = status['code']
             return STATUSES.get(code, "UNKNOWN")
-        else:
-            status = get_status(resource)
-            if status['code'] != UPLOADING:
-                LOGGER.error("Wrong resource id")
-                return
-            return STATUSES[UPLOADING]
+
+        status = get_status(resource)
+        if status['code'] != UPLOADING:
+            LOGGER.error("Wrong resource id")
+            return
+        return STATUSES[UPLOADING]
 
     def check_resource(self, resource,
                        query_string='', wait_time=1):
@@ -545,7 +641,7 @@ class BigML(ExternalConnectorHandler,
         if self.storage is not None:
             try:
                 stored_resource = os.path.join(self.storage,
-                                              resource_id.replace("/", "_"))
+                                               resource_id.replace("/", "_"))
                 with open(stored_resource) as resource_file:
                     resource = json.loads(resource_file.read())
                 # we check that the stored resource has the information
@@ -553,7 +649,8 @@ class BigML(ExternalConnectorHandler,
                 if check_local_fn is None or check_local_fn(resource):
                     return resource
             except ValueError:
-                raise ValueError("The file %s contains no JSON")
+                raise ValueError("The file %s contains no JSON" %
+                    stored_resource)
             except IOError:
                 pass
         if self.auth == '?username=;api_key=;':
@@ -562,7 +659,7 @@ class BigML(ExternalConnectorHandler,
                              " for the first time and store it locally for further"
                              " use. Please export BIGML_USERNAME"
                              " and BIGML_API_KEY."  % resource_id)
-        api_getter = self.getters[get_resource_type(resource_id)]
+
         resource = check_resource(resource_id, query_string=query_string,
                                   api=self, retries=retries)
         return resource
