@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
 #
-# Copyright 2015-2020 BigML
+# Copyright 2015-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -42,13 +41,13 @@ import csv
 
 
 from bigml.api import FINISHED
-from bigml.api import get_status, get_api_connection
+from bigml.api import get_status, get_api_connection, get_association_id
 from bigml.basemodel import get_resource_dict
 from bigml.modelfields import ModelFields
 from bigml.associationrule import AssociationRule
 from bigml.item import Item
 from bigml.io import UnicodeWriter
-
+from bigml.util import use_cache, load, dump, dumps
 
 LOGGER = logging.getLogger('BigML')
 
@@ -107,11 +106,22 @@ class Association(ModelFields):
 
     """
 
-    def __init__(self, association, api=None):
+    def __init__(self, association, api=None, cache_get=None):
+
+
+        if use_cache(cache_get):
+            # using a cache to store the association attributes
+            self.__dict__ = load(get_association_id(association), cache_get)
+            for index, item in enumerate(self.items):
+                self.items[index] = Item(item["index"], item, self.fields)
+            for index, rule in enumerate(self.rules):
+                self.rules[index] = AssociationRule(rule)
+            return
 
         self.resource_id = None
         self.complement = None
         self.discretization = {}
+        self.default_numeric_value = None
         self.field_discretizations = {}
         self.items = []
         self.max_k = None
@@ -123,10 +133,10 @@ class Association(ModelFields):
         self.search_strategy = DEFAULT_SEARCH_STRATEGY
         self.rules = []
         self.significance_level = None
-        self.api = get_api_connection(api)
+        api = get_api_connection(api)
 
         self.resource_id, association = get_resource_dict( \
-            association, "association", api=self.api)
+            association, "association", api=api)
 
         if 'object' in association and isinstance(association['object'], dict):
             association = association['object']
@@ -136,6 +146,8 @@ class Association(ModelFields):
             status = get_status(association)
             if 'code' in status and status['code'] == FINISHED:
                 self.input_fields = association['input_fields']
+                self.default_numeric_value = association.get( \
+                    'default_numeric_value')
                 associations = association['associations']
                 fields = associations['fields']
                 ModelFields.__init__( \
@@ -194,10 +206,10 @@ class Association(ModelFields):
         if score_by and score_by not in SCORES:
             raise ValueError("The available values of score_by are: %s" %
                              ", ".join(SCORES))
-        input_data = self.filter_input_data(input_data)
+        norm_input_data = self.filter_input_data(input_data)
         # retrieving the items in input_data
         items_indexes = [item.index for item in
-                         self.get_items(input_map=input_data)]
+                         self.get_items(input_map=norm_input_data)]
         if score_by is None:
             score_by = self.search_strategy
 
@@ -208,7 +220,7 @@ class Association(ModelFields):
             # if the rhs corresponds to a non-itemized field and this field
             # is already in input_data, don't add rhs
             if field_type in NO_ITEMS and self.items[rule.rhs[0]].field_id in \
-                    input_data:
+                    norm_input_data:
                 continue
             # if an itemized content is in input_data, don't add it to the
             # prediction
@@ -229,8 +241,8 @@ class Association(ModelFields):
                     predictions[rhs]["rules"] = []
                 predictions[rhs]["rules"].append(rule.rule_id)
         # choose the best k predictions
-        k = len(predictions.keys()) if k is None else k
-        predictions = sorted(predictions.items(),
+        k = len(predictions) if k is None else k
+        predictions = sorted(list(predictions.items()),
                              key=lambda x: x[1]["score"], reverse=True)[:k]
         final_predictions = []
         for rhs, prediction in predictions:
@@ -364,7 +376,7 @@ class Association(ModelFields):
                 return True
             if isinstance(item_list[0], Item):
                 items = [item.index for item in item_list]
-            elif isinstance(item_list[0], basestring):
+            elif isinstance(item_list[0], str):
                 items = [item.index for item
                          in self.get_items(names=item_list)]
 
@@ -398,7 +410,7 @@ class Association(ModelFields):
         with UnicodeWriter(file_name, quoting=csv.QUOTE_NONNUMERIC) as writer:
             writer.writerow(RULE_HEADERS)
             for rule in rules:
-                writer.writerow([item if not isinstance(item, basestring)
+                writer.writerow([item if not isinstance(item, str)
                                  else item.encode("utf-8")
                                  for item in rule])
 
@@ -415,7 +427,8 @@ class Association(ModelFields):
                 item = self.items[item_index]
                 # if there's just one field, we don't use the item description
                 # to avoid repeating the field name constantly.
-                item_description = item.name if len(self.fields.keys()) == 1 \
+                item_description = item.name if \
+                    len(list(self.fields.keys())) == 1 \
                     and not item.complement else item.describe()
                 description.append(item_description)
             description_str = " & ".join(description)
@@ -466,3 +479,26 @@ class Association(ModelFields):
                         break
             out.write("\n".join(out_rules))
         out.write("\n")
+
+    def dump(self, output=None, cache_set=None):
+        """Uses msgpack to serialize the resource object
+        If cache_set is filled with a cache set method, the method is called
+
+        """
+        self_vars = vars(self)
+        for index, elem in enumerate(self_vars["items"]):
+            self_vars["items"][index] = vars(elem)
+        for index, elem in enumerate(self_vars["rules"]):
+            self_vars["rules"][index] = vars(elem)
+        dump(self_vars, output=output, cache_set=cache_set)
+
+    def dumps(self):
+        """Uses msgpack to serialize the resource object to a string
+
+        """
+        self_vars = vars(self)
+        for index, elem in enumerate(self_vars["items"]):
+            self_vars["items"][index] = vars(elem)
+        for index, elem in enumerate(self_vars["rules"]):
+            self_vars["rules"][index] = vars(elem)
+        dumps(self_vars)
