@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
 #
-# Copyright 2014-2020 BigML
+# Copyright 2014-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -44,19 +43,19 @@ import sys
 import math
 import re
 import csv
+import codecs
+
 
 from bigml.api import FINISHED
-from bigml.api import get_status, BigML, get_api_connection
-from bigml.util import cast, utf8, PY3, NUMERIC
+from bigml.api import get_status, get_api_connection, get_cluster_id
+from bigml.util import cast, utf8, NUMERIC, use_cache, load, dump, dumps
 from bigml.centroid import Centroid
 from bigml.basemodel import get_resource_dict
-from bigml.model import print_distribution
+from bigml.generators.model import print_distribution
 from bigml.predicate import TM_TOKENS, TM_FULL_TERM
 from bigml.modelfields import ModelFields
 from bigml.io import UnicodeWriter
 
-if PY3:
-    import codecs
 
 
 LOGGER = logging.getLogger('BigML')
@@ -65,10 +64,9 @@ CSV_STATISTICS = ['minimum', 'mean', 'median', 'maximum', 'standard_deviation',
                   'sum', 'sum_squares', 'variance']
 INDENT = " " * 4
 INTERCENTROID_MEASURES = [('Minimum', min),
-                          ('Mean', lambda(x): sum(x)/float(len(x))),
+                          ('Mean', lambda x: sum(x)/float(len(x))),
                           ('Maximum', max)]
 GLOBAL_CLUSTER_LABEL = 'Global'
-NUMERIC_DEFAULTS = ["mean", "median", "minimum", "maximum", "zero"]
 
 
 def parse_terms(text, case_sensitive=True):
@@ -77,7 +75,7 @@ def parse_terms(text, case_sensitive=True):
     """
     if text is None:
         return []
-    expression = ur'(\b|_)([^\b_\s]+?)(\b|_)'
+    expression = r'(\b|_)([^\b_\s]+?)(\b|_)'
     pattern = re.compile(expression)
     return [match[1] if case_sensitive else match[1].lower()
             for match in re.findall(pattern, text)]
@@ -99,8 +97,7 @@ def get_unique_terms(terms, term_forms, tag_cloud):
 
     """
     extend_forms = {}
-    tag_cloud = tag_cloud.keys()
-    for term, forms in term_forms.items():
+    for term, forms in list(term_forms.items()):
         for form in forms:
             extend_forms[form] = term
         extend_forms[term] = term
@@ -113,6 +110,31 @@ def get_unique_terms(terms, term_forms, tag_cloud):
     return list(terms_set)
 
 
+def cluster_global_distance():
+    """Used to populate the intercentroid distances columns in the CSV
+       report. For now we don't want to compute real distance and just
+       display "N/A"
+    """
+    intercentroid_distance = []
+    for measure, _ in INTERCENTROID_MEASURES:
+        intercentroid_distance.append([measure, 'N/A'])
+    return intercentroid_distance
+
+
+def centroid_features(centroid, field_ids, encode=True):
+    """Returns features defining the centroid according to the list
+       of common field ids that define the centroids.
+
+    """
+    features = []
+    for field_id in field_ids:
+        value = centroid.center[field_id]
+        if isinstance(value, str) and encode:
+            value = utf8(value)
+        features.append(value)
+    return features
+
+
 class Cluster(ModelFields):
     """ A lightweight wrapper around a cluster model.
 
@@ -121,10 +143,20 @@ class Cluster(ModelFields):
 
     """
 
-    def __init__(self, cluster, api=None):
+    def __init__(self, cluster, api=None, cache_get=None):
+
+        self.api = get_api_connection(api)
+        self.centroids = None
+        if use_cache(cache_get):
+            # using a cache to store the cluster attributes
+            self.__dict__ = load(get_cluster_id(cluster), cache_get)
+
+            for index, centroid in enumerate(self.centroids):
+                self.centroids[index] = Centroid(centroid)
+            self.cluster_global = Centroid(self.cluster_global)
+            return
 
         self.resource_id = None
-        self.centroids = None
         self.cluster_global = None
         self.total_ss = None
         self.within_ss = None
@@ -132,6 +164,7 @@ class Cluster(ModelFields):
         self.ratio_ss = None
         self.critical_value = None
         self.input_fields = []
+        self.default_numeric_value = None
         self.summary_fields = []
         self.default_numeric_value = None
         self.k = None
@@ -143,7 +176,6 @@ class Cluster(ModelFields):
         self.item_analysis = {}
         self.items = {}
         self.datasets = {}
-        self.api = get_api_connection(api)
 
         self.resource_id, cluster = get_resource_dict( \
             cluster, "cluster", api=self.api)
@@ -192,25 +224,6 @@ class Cluster(ModelFields):
                         # clusters retrieved from API will only contain
                         # model fields
                         pass
-                for field_id, field in fields.items():
-                    if field['optype'] == 'text':
-                        self.term_forms[field_id] = {}
-                        self.term_forms[field_id].update(field[
-                            'summary']['term_forms'])
-                        self.tag_clouds[field_id] = {}
-                        self.tag_clouds[field_id].update(field[
-                            'summary']['tag_cloud'])
-                        self.term_analysis[field_id] = {}
-                        self.term_analysis[field_id].update(
-                            field['term_analysis'])
-                    if field['optype'] == 'items':
-                        self.items[field_id] = {}
-                        self.items[field_id].update(
-                            dict(field['summary']['items']))
-                        self.item_analysis[field_id] = {}
-                        self.item_analysis[field_id].update(
-                            field['item_analysis'])
-
                 missing_tokens = cluster['clusters'].get('missing_tokens')
                 ModelFields.__init__(self, fields,
                                      missing_tokens=missing_tokens)
@@ -253,21 +266,22 @@ class Cluster(ModelFields):
         """
         return self.critical_value is not None
 
-    def fill_numeric_defaults(self, input_data, average="mean"):
+    def fill_numeric_defaults(self, input_data):
         """Checks whether input data is missing a numeric field and
-        fills it with the average quantity provided in the
-        ``average`` parameter
+        fills it with the average quantity set in default_numeric_value
+
         """
 
         for field_id, field in self.fields.items():
             if (field_id not in self.summary_fields and \
                     field['optype'] == NUMERIC and
                     field_id not in input_data):
-                if average not in NUMERIC_DEFAULTS:
-                    raise ValueError("The available defaults are: %s" % \
-                 ", ".join(NUMERIC_DEFAULTS))
-                default_value = 0 if average == "zero" \
-                    else field['summary'].get(average)
+                if self.default_numeric_value is None:
+                    raise Exception("Missing values in input data. Input"
+                                    " data must contain values for all "
+                                    "numeric fields to compute a distance.")
+                default_value = 0 if self.default_numeric_value == "zero" \
+                    else field['summary'].get(self.default_numeric_value)
                 input_data[field_id] = default_value
         return input_data
 
@@ -280,7 +294,7 @@ class Cluster(ModelFields):
         for field_id in self.term_forms:
             if field_id in input_data:
                 input_data_field = input_data.get(field_id, '')
-                if isinstance(input_data_field, basestring):
+                if isinstance(input_data_field, str):
                     case_sensitive = self.term_analysis[field_id].get(
                         'case_sensitive', True)
                     token_mode = self.term_analysis[field_id].get(
@@ -295,7 +309,7 @@ class Cluster(ModelFields):
                             input_data_field if case_sensitive
                             else input_data_field.lower())
                     unique_terms[field_id] = get_unique_terms(
-                        terms, self.term_forms[field_id],
+                        terms, self.fields[field_id]["summary"]["term_forms"],
                         self.tag_clouds.get(field_id, []))
                 else:
                     unique_terms[field_id] = input_data_field
@@ -304,14 +318,14 @@ class Cluster(ModelFields):
         for field_id in self.item_analysis:
             if field_id in input_data:
                 input_data_field = input_data.get(field_id, '')
-                if isinstance(input_data_field, basestring):
+                if isinstance(input_data_field, str):
                     # parsing the items in input_data
                     separator = self.item_analysis[field_id].get(
                         'separator', ' ')
                     regexp = self.item_analysis[field_id].get(
                         'separator_regexp')
                     if regexp is None:
-                        regexp = ur'%s' % re.escape(separator)
+                        regexp = r'%s' % re.escape(separator)
                     terms = parse_items(input_data_field, regexp)
                     unique_terms[field_id] = get_unique_terms(
                         terms, {},
@@ -342,38 +356,19 @@ class Cluster(ModelFields):
             intercentroid_distance.append([measure, result])
         return intercentroid_distance
 
-    def cluster_global_distance(self):
-        """Used to populate the intercentroid distances columns in the CSV
-           report. For now we don't want to compute real distance and jsut
-           display "N/A"
-        """
-        intercentroid_distance = []
-        for measure, _ in INTERCENTROID_MEASURES:
-            intercentroid_distance.append([measure, 'N/A'])
-        return intercentroid_distance
-
     def _prepare_for_distance(self, input_data):
         """Prepares the fields to be able to compute the distance2
 
         """
         # Checks and cleans input_data leaving the fields used in the model
-        clean_input_data = self.filter_input_data(input_data)
-
-        # Checks that all numeric fields are present in input data and
-        # fills them with the default average (if given) when otherwise
-        try:
-            self.fill_numeric_defaults(clean_input_data,
-                                       self.default_numeric_value)
-        except ValueError:
-            raise Exception("Missing values in input data. Input"
-                            " data must contain values for all "
-                            "numeric fields to compute a distance.")
+        # and adding default numeric values if set
+        norm_input_data = self.filter_input_data(input_data)
         # Strips affixes for numeric values and casts to the final field type
-        cast(clean_input_data, self.fields)
+        cast(norm_input_data, self.fields)
 
-        unique_terms = self.get_unique_terms(clean_input_data)
+        unique_terms = self.get_unique_terms(norm_input_data)
 
-        return clean_input_data, unique_terms
+        return norm_input_data, unique_terms
 
     def distances2_to_point(self, reference_point,
                             list_of_points):
@@ -428,9 +423,8 @@ class Cluster(ModelFields):
         # download dataset to compute local predictions
         downloaded_data = self.api.download_dataset( \
             centroid_dataset["resource"])
-        if PY3:
-            text_reader = codecs.getreader("utf-8")
-            downloaded_data = text_reader(downloaded_data)
+        text_reader = codecs.getreader("utf-8")
+        downloaded_data = text_reader(downloaded_data)
         reader = csv.DictReader(downloaded_data)
         points = []
         for row in reader:
@@ -484,19 +478,6 @@ class Cluster(ModelFields):
                 "centroids": sorted(close_centroids,
                                     key=lambda x: x["distance"])}
 
-    def centroid_features(self, centroid, field_ids, encode=True):
-        """Returns features defining the centroid according to the list
-           of common field ids that define the centroids.
-
-        """
-        features = []
-        for field_id in field_ids:
-            value = centroid.center[field_id]
-            if isinstance(value, basestring) and encode:
-                value = value.encode('utf-8')
-            features.append(value)
-        return features
-
     def get_data_distribution(self):
         """Returns training data distribution
 
@@ -510,9 +491,9 @@ class Cluster(ModelFields):
         """Prints the line Global: 100% (<total> instances)
 
          """
-        output = u""
+        output = ""
         if self.cluster_global:
-            output += (u"    %s: 100%% (%d instances)\n" % (
+            output += ("    %s: 100%% (%d instances)\n" % (
                 self.cluster_global.name,
                 self.cluster_global.count))
         out.write(output)
@@ -527,11 +508,11 @@ class Cluster(ModelFields):
                        "of squares)", self.within_ss),
                       ("between_ss (Between sum of squares)", self.between_ss),
                       ("ratio_ss (Ratio of sum of squares)", self.ratio_ss)]
-        output = u""
+        output = ""
 
         for metric in ss_metrics:
             if metric[1]:
-                output += (u"%s%s: %5f\n" % (INDENT, metric[0], metric[1]))
+                output += ("%s%s: %5f\n" % (INDENT, metric[0], metric[1]))
 
         out.write(output)
         out.flush()
@@ -543,10 +524,10 @@ class Cluster(ModelFields):
         rows = []
         writer = None
         field_ids = self.centroids[0].center.keys()
-        headers = [u"Centroid_name"]
-        headers.extend([u"%s" % self.fields[field_id]["name"]
+        headers = ["Centroid_name"]
+        headers.extend(["%s" % self.fields[field_id]["name"]
                         for field_id in field_ids])
-        headers.extend([u"Instances"])
+        headers.extend(["Instances"])
         intercentroids = False
         header_complete = False
 
@@ -554,20 +535,20 @@ class Cluster(ModelFields):
         centroids_list = sorted(self.centroids, key=lambda x: x.name)
         for centroid in centroids_list:
             row = [centroid.name]
-            row.extend(self.centroid_features(centroid, field_ids,
-                                              encode=False))
+            row.extend(centroid_features(centroid, field_ids,
+                                         encode=False))
             row.append(centroid.count)
             if len(self.centroids) > 1:
                 for measure, result in self.centroids_distance(centroid):
                     if not intercentroids:
-                        headers.append(u"%s intercentroid distance" % \
+                        headers.append("%s intercentroid distance" % \
                             measure.title())
                     row.append(result)
                 intercentroids = True
             for measure, result in centroid.distance.items():
                 if measure in CSV_STATISTICS:
                     if not header_complete:
-                        headers.append(u"Distance %s" %
+                        headers.append("Distance %s" %
                                        measure.lower().replace("_", " "))
                     row.append(result)
             if not header_complete:
@@ -576,12 +557,12 @@ class Cluster(ModelFields):
             rows.append(row)
 
         if self.cluster_global:
-            row = [u"%s" % self.cluster_global.name]
-            row.extend(self.centroid_features(self.cluster_global, field_ids,
-                                              encode=False))
+            row = ["%s" % self.cluster_global.name]
+            row.extend(centroid_features(self.cluster_global, field_ids,
+                                         encode=False))
             row.append(self.cluster_global.count)
             if len(self.centroids) > 1:
-                for measure, result in self.cluster_global_distance():
+                for measure, result in cluster_global_distance():
                     row.append(result)
             for measure, result in self.cluster_global.distance.items():
                 if measure in CSV_STATISTICS:
@@ -601,51 +582,74 @@ class Cluster(ModelFields):
         report_header = ''
         if self.is_g_means:
             report_header = \
-                u'G-means Cluster (critical_value=%d)' % self.critical_value
+                'G-means Cluster (critical_value=%d)' % self.critical_value
         else:
-            report_header = u'K-means Cluster (k=%d)' % self.k
+            report_header = 'K-means Cluster (k=%d)' % self.k
 
         out.write(report_header + ' with %d centroids\n\n' %
                   len(self.centroids))
 
-        out.write(u"Data distribution:\n")
+        out.write("Data distribution:\n")
         # "Global" is set as first entry
         self.print_global_distribution(out=out)
         print_distribution(self.get_data_distribution(), out=out)
-        out.write(u"\n")
+        out.write("\n")
         centroids_list = [self.cluster_global] if self.cluster_global else []
         centroids_list.extend(sorted(self.centroids, key=lambda x: x.name))
 
-        out.write(u"Cluster metrics:\n")
+        out.write("Cluster metrics:\n")
         self.print_ss_metrics(out=out)
-        out.write(u"\n")
+        out.write("\n")
 
 
-        out.write(u"Centroids:\n")
+        out.write("Centroids:\n")
         for centroid in centroids_list:
-            out.write(utf8(u"\n%s%s: " % (INDENT, centroid.name)))
+            out.write(utf8("\n%s%s: " % (INDENT, centroid.name)))
             connector = ""
             for field_id, value in centroid.center.items():
-                if isinstance(value, basestring):
-                    value = u"\"%s\"" % value
-                out.write(utf8(u"%s%s: %s" % (connector,
-                                              self.fields[field_id]['name'],
-                                              value)))
+                if isinstance(value, str):
+                    value = "\"%s\"" % value
+                out.write(utf8("%s%s: %s" % (connector,
+                                             self.fields[field_id]['name'],
+                                             value)))
                 connector = ", "
-        out.write(u"\n\n")
+        out.write("\n\n")
 
-        out.write(u"Distance distribution:\n\n")
+        out.write("Distance distribution:\n\n")
         for centroid in centroids_list:
             centroid.print_statistics(out=out)
-        out.write(u"\n")
+        out.write("\n")
 
         if len(self.centroids) > 1:
-            out.write(u"Intercentroid distance:\n\n")
+            out.write("Intercentroid distance:\n\n")
             centroids_list = (centroids_list[1:] if self.cluster_global else
                               centroids_list)
             for centroid in centroids_list:
-                out.write(utf8(u"%sTo centroid: %s\n" % (INDENT,
-                                                         centroid.name)))
+                out.write(utf8("%sTo centroid: %s\n" % (INDENT,
+                                                        centroid.name)))
                 for measure, result in self.centroids_distance(centroid):
-                    out.write(u"%s%s: %s\n" % (INDENT * 2, measure, result))
-                out.write(u"\n")
+                    out.write("%s%s: %s\n" % (INDENT * 2, measure, result))
+                out.write("\n")
+
+    def dump(self, output=None, cache_set=None):
+        """Uses msgpack to serialize the resource object
+        If cache_set is filled with a cache set method, the method is called
+
+        """
+        self_vars = vars(self)
+        for index, centroid in enumerate(self_vars["centroids"]):
+            self_vars["centroids"][index] = vars(centroid)
+        self_vars["cluster_global"] = vars(self_vars["cluster_global"])
+        del self_vars["api"]
+        dump(self_vars, output=output, cache_set=cache_set)
+
+    def dumps(self):
+        """Uses msgpack to serialize the resource object to a string
+
+        """
+        self_vars = vars(self)
+        for index, centroid in enumerate(self_vars["centroids"]):
+            self_vars["centroids"][index] = vars(centroid)
+        self_vars["cluster_global"] = vars(self_vars["cluster_global"])
+        del self_vars["api"]
+        dumps(self_vars)
