@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
 #
-# Copyright 2012-2020 BigML
+# Copyright 2012-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -46,18 +45,20 @@ import os
 
 from functools import cmp_to_key
 
+from bigml.exceptions import NoRootDecisionTree
 from bigml.api import get_ensemble_id, get_model_id, get_api_connection
-from bigml.model import Model, print_distribution, \
-    parse_operating_point, sort_categories
+from bigml.model import Model, parse_operating_point, sort_categories
+from bigml.generators.model import print_distribution
 from bigml.basemodel import retrieve_resource, ONLY_MODEL, EXCLUDE_FIELDS
 from bigml.model import LAST_PREDICTION
 from bigml.multivote import MultiVote
 from bigml.multivote import PLURALITY_CODE, PROBABILITY_CODE, CONFIDENCE_CODE
 from bigml.multimodel import MultiModel
-from bigml.basemodel import BaseModel, print_importance
-from bigml.modelfields import ModelFields, lacks_info
+from bigml.basemodel import BaseModel, print_importance, check_local_but_fields
+from bigml.modelfields import ModelFields, NUMERIC
 from bigml.multivotelist import MultiVoteList
-from bigml.util import cast
+from bigml.tree_utils import add_distribution
+from bigml.util import cast, use_cache, load, dump, dumps
 
 
 BOOSTING = 1
@@ -65,14 +66,6 @@ LOGGER = logging.getLogger('BigML')
 OPERATING_POINT_KINDS = ["probability", "confidence", "votes"]
 
 
-def use_cache(cache_get):
-    """Checks whether the user has provided a cache get function to retrieve
-       local models.
-
-    """
-    return cache_get is not None and hasattr(cache_get, '__call__')
-
-
 def boosted_list_error(boosting):
     """The local ensemble cannot be built from a list of boosted models
 
@@ -104,28 +97,46 @@ class Ensemble(ModelFields):
                   cache storage.
     """
 
-    def __init__(self, ensemble,
-                 api=None,
-                 max_models=None,
-                 cache_get=None):
+    def __init__(self, ensemble, api=None, max_models=None, cache_get=None):
 
+        self.model_splits = []
+        self.multi_model = None
+        self.api = get_api_connection(api)
+        self.fields = None
+        self.class_names = None
+        self.default_numeric_value = None
+        if use_cache(cache_get):
+            # using a cache to store the model attributes
+            self.__dict__ = load(get_ensemble_id(ensemble), cache_get)
+            self.api = get_api_connection(api)
+            if len(self.models_splits) == 1:
+                # retrieve the models from a cache get function
+                try:
+                    models = [Model(model_id, cache_get=cache_get)
+                              for model_id
+                              in self.models_splits[0]]
+                except Exception as exc:
+                    raise Exception('Error while calling the user-given'
+                                    ' function %s: %s' %
+                                    (cache_get.__name__, str(exc)))
+                self.multi_model = MultiModel(models,
+                                              self.api,
+                                              fields=self.fields,
+                                              class_names=self.class_names,
+                                              cache_get=cache_get)
+            return
 
         self.resource_id = None
         self.objective_id = None
         self.distributions = None
         self.distribution = None
-        self.models_splits = []
-        self.multi_model = None
         self.boosting = None
         self.boosting_offsets = None
         self.cache_get = None
         self.regression = False
-        self.fields = None
-        self.class_names = None
         self.importance = {}
         query_string = ONLY_MODEL
         no_check_fields = False
-        self.api = get_api_connection(api)
         self.input_fields = []
         if isinstance(ensemble, list):
             if all([isinstance(model, Model) for model in ensemble]):
@@ -136,7 +147,7 @@ class Ensemble(ModelFields):
                 try:
                     models = [get_model_id(model) for model in ensemble]
                     self.model_ids = models
-                except ValueError, exc:
+                except ValueError as exc:
                     raise ValueError('Failed to verify the list of models.'
                                      ' Check your model id values: %s' %
                                      str(exc))
@@ -144,11 +155,11 @@ class Ensemble(ModelFields):
         else:
             ensemble = self.get_ensemble_resource(ensemble)
             self.resource_id = get_ensemble_id(ensemble)
-
-            if lacks_info(ensemble, inner_key="ensemble"):
+            if not check_local_but_fields(ensemble):
                 # avoid checking fields because of old ensembles
                 ensemble = retrieve_resource(self.api, self.resource_id,
                                              no_check_fields=True)
+
             if ensemble['object'].get('type') == BOOSTING:
                 self.boosting = ensemble['object'].get('boosting')
             models = ensemble['object']['models']
@@ -163,6 +174,7 @@ class Ensemble(ModelFields):
                 query_string = EXCLUDE_FIELDS
                 no_check_fields = True
             self.input_fields = ensemble['object'].get('input_fields')
+            self.default_numeric_value = ensemble.get('default_numeric_value')
 
         number_of_models = len(models)
         if max_models is None:
@@ -175,10 +187,11 @@ class Ensemble(ModelFields):
                 if use_cache(cache_get):
                     # retrieve the models from a cache get function
                     try:
-                        models = [cache_get(model_id) for model_id
+                        models = [Model(model_id, cache_get=cache_get)
+                                  for model_id
                                   in self.models_splits[0]]
                         self.cache_get = cache_get
-                    except Exception, exc:
+                    except Exception as exc:
                         raise Exception('Error while calling the user-given'
                                         ' function %s: %s' %
                                         (cache_get.__name__, str(exc)))
@@ -198,9 +211,10 @@ class Ensemble(ModelFields):
                 if use_cache(cache_get):
                     # retrieve the models from a cache get function
                     try:
-                        model = cache_get(self.models_splits[0][0])
+                        model = Model(self.models_splits[0][0],
+                                      cache_get=cache_get)
                         self.cache_get = cache_get
-                    except Exception, exc:
+                    except Exception as exc:
                         raise Exception('Error while calling the user-given'
                                         ' function %s: %s' %
                                         (cache_get.__name__, str(exc)))
@@ -218,7 +232,7 @@ class Ensemble(ModelFields):
                 self.distributions = []
                 for model in models:
                     self.distributions.append({
-                        'training': {'categories': model.tree.distribution}
+                        'training': model.root_distribution
                     })
             except AttributeError:
                 self.distributions = [model['object']['model']['distribution']
@@ -232,19 +246,9 @@ class Ensemble(ModelFields):
                 max_models=max_models)
 
         if self.fields:
-            summary = self.fields[self.objective_id]['summary']
-            if 'bins' in summary:
-                distribution = summary['bins']
-            elif 'counts' in summary:
-                distribution = summary['counts']
-            elif 'categories' in summary:
-                distribution = summary['categories']
-            else:
-                distribution = []
-            self.distribution = distribution
-
+            add_distribution(self)
         self.regression = \
-            self.fields[self.objective_id].get('optype') == 'numeric'
+            self.fields[self.objective_id].get('optype') == NUMERIC
         if self.boosting:
             self.boosting_offsets = ensemble['object'].get('initial_offset',
                                                            0) \
@@ -269,11 +273,14 @@ class Ensemble(ModelFields):
         ModelFields.__init__( \
             self, self.fields,
             objective_id=self.objective_id)
+
         if len(self.models_splits) == 1:
             self.multi_model = MultiModel(models,
                                           self.api,
                                           fields=self.fields,
                                           class_names=self.class_names)
+            for index, model in enumerate(self.multi_model.models):
+                self.multi_model.models[index].term_forms = self.term_forms
 
     def _add_models_attrs(self, model, max_models=None):
         """ Adds the boosting and fields info when the ensemble is built from
@@ -300,7 +307,7 @@ class Ensemble(ModelFields):
            - an ensemble id
         """
         # the string can be a path to a JSON file
-        if isinstance(ensemble, basestring):
+        if isinstance(ensemble, str):
             try:
                 path = os.path.dirname(os.path.abspath(ensemble))
                 with open(ensemble) as ensemble_file:
@@ -310,8 +317,7 @@ class Ensemble(ModelFields):
                         raise ValueError("The JSON file does not seem"
                                          " to contain a valid BigML ensemble"
                                          " representation.")
-                    else:
-                        self.api.storage = path
+                    self.api.storage = path
             except IOError:
                 # if it is not a path, it can be an ensemble id
                 self.resource_id = get_ensemble_id(ensemble)
@@ -321,9 +327,8 @@ class Ensemble(ModelFields):
                             self.api.error_message(ensemble,
                                                    resource_type='ensemble',
                                                    method='get'))
-                    else:
-                        raise IOError("Failed to open the expected JSON file"
-                                      " at %s" % ensemble)
+                    raise IOError("Failed to open the expected JSON file"
+                                  " at %s" % ensemble)
             except ValueError:
                 raise ValueError("Failed to interpret %s."
                                  " JSON file expected.")
@@ -384,7 +389,7 @@ class Ensemble(ModelFields):
                 missing_strategy)
 
             if not compact:
-                names_probabilities = zip(self.class_names, output)
+                names_probabilities = list(zip(self.class_names, output))
                 output = [{'category': class_name,
                            'probability': probability}
                           for class_name, probability in names_probabilities]
@@ -432,7 +437,7 @@ class Ensemble(ModelFields):
                 missing_strategy,
                 method=CONFIDENCE_CODE)
             if not compact:
-                names_confidences = zip(self.class_names, output)
+                names_confidences = list(zip(self.class_names, output))
                 output = [{'category': class_name,
                            'confidence': confidence}
                           for class_name, confidence in names_confidences]
@@ -478,7 +483,7 @@ class Ensemble(ModelFields):
                 missing_strategy,
                 method=PLURALITY_CODE)
             if not compact:
-                names_votes = zip(self.class_names, output)
+                names_votes = list(zip(self.class_names, output))
                 output = [{'category': class_name,
                            'votes': k}
                           for class_name, k in names_votes]
@@ -504,6 +509,8 @@ class Ensemble(ModelFields):
                                          api=self.api,
                                          fields=self.fields,
                                          class_names=self.class_names)
+                for index, _ in enumerate(multi_model.models):
+                    multi_model.models[index].term_forms = self.term_forms
 
                 votes_split = multi_model.generate_votes_distribution( \
                     input_data,
@@ -527,7 +534,7 @@ class Ensemble(ModelFields):
                 try:
                     models = [self.cache_get(model_id) for model_id
                               in models_split]
-                except Exception, exc:
+                except Exception as exc:
                     raise Exception('Error while calling the '
                                     'user-given'
                                     ' function %s: %s' %
@@ -689,17 +696,15 @@ class Ensemble(ModelFields):
         """
 
         # Checks and cleans input_data leaving the fields used in the model
-        new_data = self.filter_input_data( \
+        norm_input_data = self.filter_input_data( \
             input_data,
             add_unused_fields=full)
         unused_fields = None
         if full:
-            input_data, unused_fields = new_data
-        else:
-            input_data = new_data
+            norm_input_data, unused_fields = norm_input_data
 
         # Strips affixes for numeric values and casts to the final field type
-        cast(input_data, self.fields)
+        cast(norm_input_data, self.fields)
 
         if median and method is None:
             # predictions with median are only available with old combiners
@@ -716,13 +721,12 @@ class Ensemble(ModelFields):
                 raise ValueError("The operating_point argument can only be"
                                  " used in classifications.")
             prediction = self.predict_operating( \
-                input_data,
+                norm_input_data,
                 missing_strategy=missing_strategy,
                 operating_point=operating_point)
             if full:
                 return prediction
-            else:
-                return prediction["prediction"]
+            return prediction["prediction"]
 
         if operating_kind:
             if self.regression:
@@ -730,15 +734,14 @@ class Ensemble(ModelFields):
                 # combiners
                 method = 1 if operating_kind == "confidence" else 0
                 return self.predict( \
-                    input_data, method=method,
+                    norm_input_data, method=method,
                     options=options, missing_strategy=missing_strategy,
                     operating_point=None, operating_kind=None, full=full)
-            else:
-                prediction = self.predict_operating_kind( \
-                    input_data,
-                    missing_strategy=missing_strategy,
-                    operating_kind=operating_kind)
-                return prediction
+            prediction = self.predict_operating_kind( \
+                norm_input_data,
+                missing_strategy=missing_strategy,
+                operating_kind=operating_kind)
+            return prediction
 
         if len(self.models_splits) > 1:
             # If there's more than one chunk of models, they must be
@@ -750,9 +753,11 @@ class Ensemble(ModelFields):
                 multi_model = MultiModel(models,
                                          api=self.api,
                                          fields=self.fields)
+                for index, _ in enumerate(multi_model.models):
+                    multi_model.models[index].term_forms = self.term_forms
 
                 votes_split = multi_model._generate_votes(
-                    input_data,
+                    norm_input_data,
                     missing_strategy=missing_strategy,
                     unused_fields=unused_fields)
                 if median:
@@ -763,7 +768,7 @@ class Ensemble(ModelFields):
             # When only one group of models is found you use the
             # corresponding multimodel to predict
             votes_split = self.multi_model._generate_votes(
-                input_data, missing_strategy=missing_strategy,
+                norm_input_data, missing_strategy=missing_strategy,
                 unused_fields=unused_fields)
 
             votes = MultiVote(votes_split.predictions,
@@ -771,6 +776,7 @@ class Ensemble(ModelFields):
             if median:
                 for prediction in votes.predictions:
                     prediction['prediction'] = prediction['median']
+
         if self.boosting is not None and not self.regression:
             categories = [ \
                 d[0] for d in
@@ -778,7 +784,7 @@ class Ensemble(ModelFields):
             options = {"categories": categories}
         result = votes.combine(method=method, options=options, full=full)
         if full:
-            unused_fields = set(input_data.keys())
+            unused_fields = set(norm_input_data.keys())
             for prediction in votes.predictions:
                 unused_fields = unused_fields.intersection( \
                     set(prediction.get("unused_fields", [])))
@@ -798,9 +804,9 @@ class Ensemble(ModelFields):
         if self.importance:
             field_importance = self.importance
             field_names = {field_id: {'name': self.fields[field_id]["name"]} \
-                           for field_id in field_importance.keys()}
+                           for field_id in list(field_importance.keys())}
             return [list(importance) for importance in \
-                sorted(field_importance.items(), key=lambda x: x[1],
+                sorted(list(field_importance.items()), key=lambda x: x[1],
                        reverse=True)], field_names
 
         if (self.distributions is not None and
@@ -809,8 +815,7 @@ class Ensemble(ModelFields):
             # Extracts importance from ensemble information
             importances = [model_info['importance'] for model_info in
                            self.distributions]
-            for index in range(0, len(importances)):
-                model_info = importances[index]
+            for model_info in importances:
                 for field_info in model_info:
                     field_id = field_info[0]
                     if field_id not in field_importance:
@@ -834,7 +839,7 @@ class Ensemble(ModelFields):
         for field_id in field_importance:
             field_importance[field_id] /= number_of_models
         return [list(importance) for importance in \
-            sorted(field_importance.items(), key=lambda x: x[1],
+            sorted(list(field_importance.items()), key=lambda x: x[1],
                    reverse=True)], field_names
 
     def print_importance(self, out=sys.stdout):
@@ -882,19 +887,19 @@ class Ensemble(ModelFields):
         distribution = self.get_data_distribution("training")
 
         if distribution:
-            out.write(u"Data distribution:\n")
+            out.write("Data distribution:\n")
             print_distribution(distribution, out=out)
-            out.write(u"\n\n")
+            out.write("\n\n")
 
         if not self.boosting:
             predictions = self.get_data_distribution("predictions")
 
             if predictions:
-                out.write(u"Predicted distribution:\n")
+                out.write("Predicted distribution:\n")
                 print_distribution(predictions, out=out)
-                out.write(u"\n\n")
+                out.write("\n\n")
 
-        out.write(u"Field importance:\n")
+        out.write("Field importance:\n")
         self.print_importance(out=out)
         out.flush()
 
@@ -913,24 +918,51 @@ class Ensemble(ModelFields):
                 models.extend(split)
         else:
             models = self.model_ids
+
         for index, model_id in enumerate(models):
-            if isinstance(model_id, Model):
-                local_model = model_id
-            elif self.cache_get is not None:
-                local_model = self.cache_get(model_id)
-            else:
-                local_model = Model(model_id, self.api)
-            if (max_models is not None and index > 0 and
-                    index % max_models == 0):
-                gc.collect()
-            fields.update(local_model.fields)
-            if (objective_id is not None and
-                    objective_id != local_model.objective_id):
-                # the models' objective field have different ids, no global id
-                no_objective_id = True
-            else:
-                objective_id = local_model.objective_id
+            try:
+                if isinstance(model_id, Model):
+                    local_model = model_id
+                elif self.cache_get is not None:
+                    local_model = self.cache_get(model_id)
+                else:
+                    local_model = Model(model_id, self.api)
+                if (max_models is not None and index > 0 and
+                        index % max_models == 0):
+                    gc.collect()
+                fields.update(local_model.fields)
+                if (objective_id is not None and
+                        objective_id != local_model.objective_id):
+                    # the models' objective field have different ids, no global id
+                    no_objective_id = True
+                else:
+                    objective_id = local_model.objective_id
+            except NoRootDecisionTree:
+                pass
         if no_objective_id:
             objective_id = None
         gc.collect()
         return fields, objective_id
+
+    def dump(self, output=None, cache_set=None):
+        """Uses msgpack to serialize the resource object
+        If cache_set is filled with a cache set method, the method is called
+
+        """
+        self_vars = vars(self)
+        del self_vars["api"]
+        if "multi_model" in self_vars:
+            for model in self_vars["multi_model"].models:
+                model.dump(output=output, cache_set=cache_set)
+            del self_vars["multi_model"]
+        dump(self_vars, output=output, cache_set=cache_set)
+
+    def dumps(self):
+        """Uses msgpack to serialize the resource object to a string
+
+        """
+        self_vars = vars(self)
+        del self_vars["api"]
+        if "multi_model" in self_vars:
+            del self_vars["multi_model"]
+        dumps(self_vars)
