@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
 #
-# Copyright 2012-2020 BigML
+# Copyright 2012-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -51,10 +50,7 @@ from bigml.util import invert_dictionary, python_map_type, find_locale
 from bigml.util import DEFAULT_LOCALE
 from bigml.api import get_resource_type, get_fields
 from bigml.constants import (
-    SOURCE_PATH, DATASET_PATH, PREDICTION_PATH, MODEL_PATH, CLUSTER_PATH,
-    ANOMALY_PATH, SAMPLE_PATH, CORRELATION_PATH, STATISTICAL_TEST_PATH,
-    LOGISTIC_REGRESSION_PATH, ASSOCIATION_PATH, TOPIC_MODEL_PATH,
-    ENSEMBLE_PATH, PCA_PATH, LINEAR_REGRESSION_PATH, FIELDS_PARENT,
+    SOURCE_PATH, DATASET_PATH, SUPERVISED_PATHS, FUSION_PATH,
     RESOURCES_WITH_FIELDS, DEFAULT_MISSING_TOKENS)
 from bigml.io import UnicodeReader, UnicodeWriter
 
@@ -70,7 +66,7 @@ UPDATABLE_HEADERS = {"field name": "name",
                      "field type": "optype",
                      "preferred": "preferred"}
 
-ITEM_SINGULAR = {u"categories": u"category"}
+ITEM_SINGULAR = {"categories": "category"}
 
 
 def get_fields_structure(resource, errors=False):
@@ -103,7 +99,8 @@ def get_fields_structure(resource, errors=False):
                 'objective_field', {}).get('id')
             if errors:
                 field_errors = resource.get("status", {}).get("field_errors")
-        elif resource_type in [MODEL_PATH, LOGISTIC_REGRESSION_PATH]:
+        elif resource_type in SUPERVISED_PATHS and \
+                resource_type != FUSION_PATH:
             objective_id = resource.get( \
                 'objective_fields', [None])[0]
             objective_column = fields.get( \
@@ -112,9 +109,8 @@ def get_fields_structure(resource, errors=False):
         if errors:
             result = result + (field_errors,)
         return result
-    else:
-        return (None, None, None, None, None) if errors else \
-            (None, None, None, None)
+    return (None, None, None, None, None) if errors else \
+        (None, None, None, None)
 
 
 def attribute_summary(attribute_value, item_type, limit=None):
@@ -123,14 +119,14 @@ def attribute_summary(attribute_value, item_type, limit=None):
     """
     if attribute_value is None:
         return None
-    items = [u"%s (%s)" % (item, instances) for
+    items = ["%s (%s)" % (item, instances) for
              item, instances in attribute_value]
     items_length = len(items)
     if limit is None or limit > items_length:
         limit = items_length
-    return u"%s %s: %s" % (items_length, type_singular(item_type,
-                                                       items_length == 1),
-                           u", ".join(items[0: limit]))
+    return "%s %s: %s" % (items_length, type_singular(item_type,
+                                                      items_length == 1),
+                          ", ".join(items[0: limit]))
 
 def type_singular(item_type, singular=False):
     """Singularizes item types if needed
@@ -141,7 +137,7 @@ def type_singular(item_type, singular=False):
     return item_type
 
 
-class Fields(object):
+class Fields():
     """A class to deal with BigML auto-generated ids.
 
     """
@@ -154,6 +150,7 @@ class Fields(object):
         # structure. The structure is checked and fields structure is returned
         # if a resource type is matched.
         try:
+            self.resource_type = get_resource_type(resource_or_fields)
             resource_info = get_fields_structure(resource_or_fields, True)
             (self.fields,
              resource_locale,
@@ -185,7 +182,7 @@ class Fields(object):
         self.missing_tokens = missing_tokens
         self.fields_columns = sorted(self.fields_by_column_number.keys())
         # Ids of the fields to be included
-        self.filtered_fields = (self.fields.keys() if include is None
+        self.filtered_fields = (list(self.fields.keys()) if include is None
                                 else include)
         # To be updated in update_objective_field
         self.row_ids = None
@@ -211,7 +208,7 @@ class Fields(object):
         # If no objective field, select the last column, else store its column
         if objective_field is None:
             self.objective_field = self.fields_columns[-1]
-        elif isinstance(objective_field, basestring):
+        elif isinstance(objective_field, str):
             try:
                 self.objective_field = self.field_column_number( \
                     objective_field)
@@ -231,7 +228,7 @@ class Fields(object):
         if headers is None:
             # The row is supposed to contain the fields sorted by column number
             self.row_ids = [item[0] for item in
-                            sorted(self.fields.items(),
+                            sorted(list(self.fields.items()),
                                    key=lambda x: x[1]['column_number'])
                             if objective_field_present or
                             item[1]['column_number'] != self.objective_field]
@@ -255,37 +252,39 @@ class Fields(object):
 
         """
 
-        if isinstance(key, basestring):
+        if isinstance(key, str):
             try:
-                id = self.fields_by_name[key]
+                f_id = self.fields_by_name[key]
             except KeyError:
                 raise ValueError("Error: field name '%s' does not exist" % key)
-            return id
-        elif isinstance(key, int):
+            return f_id
+        if isinstance(key, int):
             try:
-                id = self.fields_by_column_number[key]
+                f_id = self.fields_by_column_number[key]
             except KeyError:
                 raise ValueError("Error: field column number '%s' does not"
                                  " exist" % key)
-            return id
+            return f_id
+        return None
 
     def field_name(self, key):
         """Returns a field name.
 
         """
-        if isinstance(key, basestring):
+        if isinstance(key, str):
             try:
                 name = self.fields[key]['name']
             except KeyError:
                 raise ValueError("Error: field id '%s' does not exist" % key)
             return name
-        elif isinstance(key, int):
+        if isinstance(key, int):
             try:
                 name = self.fields[self.fields_by_column_number[key]]['name']
             except KeyError:
                 raise ValueError("Error: field column number '%s' does not"
                                  " exist" % key)
             return name
+        return None
 
     def field_column_number(self, key):
         """Returns a field column number.
@@ -319,7 +318,7 @@ class Fields(object):
             else:
                 objective_field = self.objective_field
         # If objective fields is a name or an id, retrive column number
-        if isinstance(objective_field, basestring):
+        if isinstance(objective_field, str):
             objective_field = self.field_column_number(objective_field)
 
         # Try to guess if objective field is in the data by using headers or
@@ -346,7 +345,7 @@ class Fields(object):
 
         """
         for field in [(val['name'], val['optype'], val['column_number'])
-                      for _, val in sorted(self.fields.items(),
+                      for _, val in sorted(list(self.fields.items()),
                                            key=lambda k:
                                            k[1]['column_number'])]:
             out.write('[%-32s: %-16s: %-8s]\n' % (field[0],
@@ -358,7 +357,7 @@ class Fields(object):
            it isn't set at all.
 
         """
-        return {key: field for key, field in self.fields.iteritems()
+        return {key: field for key, field in self.fields.items()
                 if ('preferred' not in field) or field['preferred']}
 
     def validate_input_data(self, input_data, out=sys.stdout):
@@ -387,8 +386,8 @@ class Fields(object):
         """Transforms to unicode and cleans missing tokens
 
         """
-        if not isinstance(value, unicode):
-            value = unicode(value, "utf-8")
+        if not isinstance(value, str):
+            value = str(value, "utf-8")
         return None if value in self.missing_tokens else value
 
     def to_input_data(self, row):
@@ -405,7 +404,7 @@ class Fields(object):
 
         """
         summaries = [(field_id, field.get('summary', {}))
-                     for field_id, field in self.fields.items()]
+                     for field_id, field in list(self.fields.items())]
         if len(summaries) == 0:
             raise ValueError("The structure has not enough information "
                              "to extract the fields containing missing values."
@@ -413,9 +412,9 @@ class Fields(object):
                              "You could retry the get remote call "
                              " with 'limit=-1' as query string.")
 
-        return dict([(field_id, summary.get('missing_count', 0))
-                     for field_id, summary in summaries
-                     if summary.get('missing_count', 0) > 0])
+        return {field_id: summary.get('missing_count', 0)
+                for field_id, summary in summaries
+                if summary.get('missing_count', 0) > 0}
 
     def stats(self, field_name):
         """Returns the summary information for the field
@@ -460,7 +459,7 @@ class Fields(object):
             else:
                 field_summary.append(json.dumps(field.get('preferred')))
                 field_summary.append(field_summary_value.get("missing_count"))
-                if self.field_errors and field_id in self.field_errors.keys():
+                if self.field_errors and field_id in list(self.field_errors.keys()):
                     errors = self.field_errors.get(field_id)
                     field_summary.append(errors.get("total"))
                 else:
@@ -473,22 +472,22 @@ class Fields(object):
                 elif field['optype'] == 'categorical':
                     categories = field_summary_value.get("categories")
                     field_summary.append( \
-                        attribute_summary(categories, u"categorìes",
+                        attribute_summary(categories, "categorìes",
                                           limit=LIST_LIMIT))
                 elif field['optype'] == "text":
                     terms = field_summary_value.get("tag_cloud")
                     field_summary.append( \
-                        attribute_summary(terms, u"terms",
+                        attribute_summary(terms, "terms",
                                           limit=LIST_LIMIT))
                 elif field['optype'] == "items":
                     items = field_summary_value.get("items")
                     field_summary.append( \
-                        attribute_summary(items, u"items", limit=LIST_LIMIT))
+                        attribute_summary(items, "items", limit=LIST_LIMIT))
                 else:
                     field_summary.append("")
-                if self.field_errors and field_id in self.field_errors.keys():
+                if self.field_errors and field_id in list(self.field_errors.keys()):
                     field_summary.append( \
-                        attribute_summary(errors.get("sample"), u"errors",
+                        attribute_summary(errors.get("sample"), "errors",
                                           limit=None))
                 else:
                     field_summary.append("")
@@ -498,8 +497,7 @@ class Fields(object):
                 summary.append(field_summary)
         if writer is None:
             return summary
-        else:
-            writer.close_writer()
+        writer.close_writer()
 
     def new_fields_structure(self, csv_attributes_file=None,
                              attributes=None, out_file=None):
@@ -522,10 +520,11 @@ class Fields(object):
         if "field ID" in attributes[0] or "field column" in attributes[0]:
             # headers are used
             for index in range(1, len(attributes)):
-                new_attributes = dict(zip(attributes[0], attributes[index]))
+                new_attributes = dict(list(zip(attributes[0],
+                                               attributes[index])))
                 if new_attributes.get("field ID"):
                     field_id = new_attributes.get("field ID")
-                    if not field_id in self.fields.keys():
+                    if not field_id in list(self.fields.keys()):
                         raise ValueError("Field ID %s not found"
                                          " in this resource" % field_id)
                     del new_attributes["field ID"]
@@ -540,9 +539,9 @@ class Fields(object):
                                          " in this resource" % field_column)
                     field_id = self.field_id(field_column)
                     del new_attributes["field column"]
-                new_attributes_headers = new_attributes.keys()
+                new_attributes_headers = list(new_attributes.keys())
                 for attribute in new_attributes_headers:
-                    if not attribute in UPDATABLE_HEADERS.keys():
+                    if not attribute in list(UPDATABLE_HEADERS.keys()):
                         del new_attributes[attribute]
                     else:
                         new_attributes[UPDATABLE_HEADERS[attribute]] = \
@@ -575,21 +574,21 @@ class Fields(object):
                     field_id = field_attributes[0] if first_column_is_id else \
                         self.field_id(int(field_attributes[0]))
                     new_fields_structure[field_id] = \
-                        dict(zip(headers, field_attributes[1: 6]))
+                        dict(list(zip(headers, field_attributes[1: 6])))
             except ValueError:
                 raise ValueError("The first column should contain either the"
                                  " column or ID of the fields. Failed to find"
                                  " %s as either of them." % field_id)
+        new_fields_structure = {"fields": new_fields_structure}
         if out_file is None:
-            return {"fields": new_fields_structure}
-        else:
-            try:
-                with open(out_file, "w") as out:
-                    json.dump({"fields": new_fields_structure}, out)
-            except IOError:
-                raise IOError("Failed writing the fields structure file in"
-                              " %s- Please, check your arguments." %
-                              out_file)
+            return new_fields_structure
+        try:
+            with open(out_file, "w") as out:
+                json.dump(new_fields_structure, out)
+        except IOError:
+            raise IOError("Failed writing the fields structure file in"
+                          " %s- Please, check your arguments." %
+                          out_file)
 
     def training_data_example(self, missings=False):
         """Generates an example of training data based on the contents of the
@@ -599,7 +598,7 @@ class Fields(object):
 
         """
         training_data = {}
-        for field_id, field in self.fields.items():
+        for _, field in list(self.fields.items()):
             if field.get("summary") is not None:
                 value = None
                 optype = field.get("optype")
@@ -637,3 +636,20 @@ class Fields(object):
                 if value is not None:
                     training_data.update({field["name"]: value})
         return training_data
+
+    def filter_fields_update(self, update_body):
+        """Filters the updatable attributes according to the type of resource
+
+        """
+        fields_info = update_body.get("fields")
+        if self.resource_type and fields_info is not None:
+            if self.resource_type == "dataset":
+                for _, field in list(fields_info.items()):
+                    if field.get("optype") is not None:
+                        del field["optype"]
+            elif self.resource_type == "source":
+                for _, field in list(fields_info.items()):
+                    if field.get("preferred") is not None:
+                        del field["preferred"]
+            update_body["fields"] = fields_info
+        return update_body
