@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
 #
-# Copyright 2012-2020 BigML
+# Copyright 2012-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -51,9 +50,11 @@ from bigml.model import parse_operating_point, sort_categories
 from bigml.model import LAST_PREDICTION
 from bigml.basemodel import get_resource_dict, retrieve_resource
 from bigml.multivotelist import MultiVoteList
-from bigml.util import cast, check_no_missing_numerics
+from bigml.util import cast, check_no_missing_numerics, use_cache, load, \
+    dump, dumps, NUMERIC
 from bigml.supervised import SupervisedModel
 from bigml.modelfields import ModelFields
+from bigml.tree_utils import add_distribution
 
 
 
@@ -71,11 +72,11 @@ def rearrange_prediction(origin_classes, destination_classes, prediction):
     """
     new_prediction = []
     for class_name in destination_classes:
-        origin_index = origin_classes.index(class_name)
-        if origin_index > -1:
+        try:
+            origin_index = origin_classes.index(class_name)
             new_prediction.append(prediction[origin_index])
-        else:
-            new_prediction = 0.0
+        except ValueError:
+            new_prediction.append(0.0)
     return new_prediction
 
 
@@ -128,7 +129,13 @@ class Fusion(ModelFields):
                   cache storage.
     """
 
-    def __init__(self, fusion, api=None, max_models=None):
+    def __init__(self, fusion, api=None, max_models=None, cache_get=None):
+
+        if use_cache(cache_get):
+            # using a cache to store the model attributes
+            self.__dict__ = load(get_fusion_id(fusion), cache_get)
+            self.api = get_api_connection(api)
+            return
 
         self.resource_id = None
         self.models_ids = None
@@ -166,12 +173,13 @@ class Fusion(ModelFields):
         number_of_models = len(self.model_ids)
 
         # Downloading the model information to cache it
-        if self.api.storage is not None:
+        if self.api.storage is not None or cache_get is not None:
             for model_id in self.model_ids:
                 if get_resource_type(model_id) == "fusion":
-                    Fusion(model_id, api=self.api)
+                    Fusion(model_id, api=self.api, cache_get=cache_get)
                 else:
-                    SupervisedModel(model_id, api=self.api)
+                    SupervisedModel(model_id, api=self.api,
+                                    cache_get=cache_get)
 
         if max_models is None:
             self.models_splits = [self.model_ids]
@@ -180,41 +188,37 @@ class Fusion(ModelFields):
                                   for index
                                   in range(0, number_of_models, max_models)]
 
-        if self.fields:
-            summary = self.fields[self.objective_id]['summary']
-            if 'bins' in summary:
-                distribution = summary['bins']
-            elif 'counts' in summary:
-                distribution = summary['counts']
-            elif 'categories' in summary:
-                distribution = summary['categories']
-            else:
-                distribution = []
-            self.distribution = distribution
-
-        self.regression = \
-            self.fields[self.objective_id].get('optype') == 'numeric'
-
-        if not self.regression:
-            objective_field = self.fields[self.objective_id]
-            categories = objective_field['summary']['categories']
-            classes = [category[0] for category in categories]
-            self.class_names = sorted(classes)
-            self.objective_categories = [category for \
-                category, _ in self.fields[self.objective_id][ \
-               "summary"]["categories"]]
 
         ModelFields.__init__( \
             self, self.fields,
             objective_id=self.objective_id)
 
+        add_distribution(self)
+        summary = self.fields[self.objective_id]['summary']
+        if 'bins' in summary:
+            distribution = summary['bins']
+        elif 'counts' in summary:
+            distribution = summary['counts']
+        elif 'categories' in summary:
+            distribution = summary['categories']
+            self.objective_categories = [
+                category for category, _ in distribution]
+            self.class_names = sorted(
+                self.objective_categories)
+        else:
+            distribution = []
+        self.distribution = distribution
+        self.regression = \
+            self.fields[self.objective_id].get('optype') == NUMERIC
+
+
     def get_fusion_resource(self, fusion):
         """Extracts the fusion resource info. The fusion argument can be
            - a path to a local file
            - an fusion id
         """
         # the string can be a path to a JSON file
-        if isinstance(fusion, basestring):
+        if isinstance(fusion, str):
             try:
                 path = os.path.dirname(os.path.abspath(fusion))
                 with open(fusion) as fusion_file:
@@ -224,25 +228,23 @@ class Fusion(ModelFields):
                         raise ValueError("The JSON file does not seem"
                                          " to contain a valid BigML fusion"
                                          " representation.")
-                    else:
-                        self.api = BigML(storage=path)
+                    api = BigML(storage=path)
             except IOError:
                 # if it is not a path, it can be an fusion id
                 self.resource_id = get_fusion_id(fusion)
                 if self.resource_id is None:
                     if fusion.find('fusion/') > -1:
                         raise Exception(
-                            self.api.error_message(fusion,
-                                                   resource_type='fusion',
-                                                   method='get'))
-                    else:
-                        raise IOError("Failed to open the expected JSON file"
-                                      " at %s" % fusion)
+                            api.error_message(fusion,
+                                              resource_type='fusion',
+                                              method='get'))
+                    raise IOError("Failed to open the expected JSON file"
+                                  " at %s" % fusion)
             except ValueError:
                 raise ValueError("Failed to interpret %s."
                                  " JSON file expected.")
         if not isinstance(fusion, dict):
-            fusion = retrieve_resource(self.api, self.resource_id,
+            fusion = retrieve_resource(api, self.resource_id,
                                        no_check_fields=False)
         return fusion
 
@@ -322,8 +324,7 @@ class Fusion(ModelFields):
         if self.regression:
             total_weight = len(votes.predictions) if self.weights is None \
                 else sum(self.weights)
-            prediction = sum([prediction for prediction in \
-                votes.predictions]) / float(total_weight)
+            prediction = sum(votes.predictions) / float(total_weight)
             if compact:
                 output = [prediction]
             else:
@@ -366,13 +367,16 @@ class Fusion(ModelFields):
                          operating point can be defined in terms of:
                          - the positive_class, the class that is important to
                            predict accurately
-                         - the probability_threshold,
-                           the probability that is stablished
+                         - the threshold,
+                           the value that is stablished
                            as minimum for the positive_class to be predicted.
+                         - the kind of measure used to set a threshold:
+                           probability or confidence (if available)
                          The operating_point is then defined as a map with
                          two attributes, e.g.:
                            {"positive_class": "Iris-setosa",
-                            "probability_threshold": 0.5}
+                            "threshold": 0.5,
+                            "kind": "probability"}
         full: Boolean that controls whether to include the prediction's
               attributes. By default, only the prediction is produced. If set
               to True, the rest of available information is added in a
@@ -405,7 +409,7 @@ class Fusion(ModelFields):
             unused_fields=unused_fields)
         if full:
             return dict((key, value) for key, value in \
-                full_prediction.iteritems() if value is not None)
+                full_prediction.items() if value is not None)
 
         return full_prediction['prediction']
 
@@ -484,3 +488,20 @@ class Fusion(ModelFields):
         if a[criteria] == b[criteria]:
             return sort_categories(a, b, self.objective_categories)
         return 1 if b[criteria] > a[criteria] else -1
+
+    def dump(self, output=None, cache_set=None):
+        """Uses msgpack to serialize the resource object
+        If cache_set is filled with a cache set method, the method is called
+
+        """
+        self_vars = vars(self)
+        del self_vars["api"]
+        dump(self_vars, output=output, cache_set=cache_set)
+
+    def dumps(self):
+        """Uses msgpack to serialize the resource object to a string
+
+        """
+        self_vars = vars(self)
+        del self_vars["api"]
+        dumps(self_vars)
