@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
 #
-# Copyright 2015-2020 BigML
+# Copyright 2015-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -43,20 +42,21 @@ logistic_regression.predict({"petal length": 3, "petal width": 1,
 import logging
 import math
 import copy
-import json
 
 from functools import cmp_to_key
 
 from bigml.api import FINISHED
-from bigml.api import get_status, get_api_connection
-from bigml.util import cast, check_no_missing_numerics, PRECISION, NUMERIC
+from bigml.api import get_status, get_api_connection, \
+    get_logistic_regression_id
+from bigml.util import cast, check_no_missing_numerics, use_cache, load, \
+    PRECISION, NUMERIC
 from bigml.basemodel import get_resource_dict, extract_objective
 from bigml.model import parse_operating_point, sort_categories
 from bigml.modelfields import ModelFields
 
 LOGGER = logging.getLogger('BigML')
 
-EXPANSION_ATTRIBUTES = {"categorical": "categories", "text": "tag_cloud",
+EXPANSION_ATTRIBUTES = {"categorical": "categories", "text": "tag_clouds",
                         "items": "items"}
 
 
@@ -88,7 +88,13 @@ class LogisticRegression(ModelFields):
 
     """
 
-    def __init__(self, logistic_regression, api=None):
+    def __init__(self, logistic_regression, api=None, cache_get=None):
+
+        if use_cache(cache_get):
+            # using a cache to store the model attributes
+            self.__dict__ = load(get_logistic_regression_id( \
+                logistic_regression), cache_get)
+            return
 
         self.resource_id = None
         self.class_names = None
@@ -103,6 +109,7 @@ class LogisticRegression(ModelFields):
         self.data_field_types = {}
         self.field_codings = {}
         self.numeric_fields = {}
+        self.default_numeric_value = None
         self.bias = None
         self.missing_numerics = None
         self.c = None
@@ -110,18 +117,20 @@ class LogisticRegression(ModelFields):
         self.lr_normalize = None
         self.balance_fields = None
         self.regularization = None
-        self.api = get_api_connection(api)
+        api = get_api_connection(api)
 
         old_coefficients = False
 
         self.resource_id, logistic_regression = get_resource_dict( \
-            logistic_regression, "logisticregression", api=self.api)
+            logistic_regression, "logisticregression", api=api)
 
         if 'object' in logistic_regression and \
             isinstance(logistic_regression['object'], dict):
             logistic_regression = logistic_regression['object']
         try:
             self.input_fields = logistic_regression.get("input_fields", [])
+            self.default_numeric_value = logistic_regression.get(
+                "default_numeric_value")
             self.dataset_field_types = logistic_regression.get(
                 "dataset_field_types", {})
             self.weight_field = logistic_regression.get("weight_field")
@@ -142,11 +151,11 @@ class LogisticRegression(ModelFields):
                 if not self.input_fields:
                     self.input_fields = [ \
                         field_id for field_id, _ in
-                        sorted(fields.items(),
+                        sorted(list(fields.items()),
                                key=lambda x: x[1].get("column_number"))]
                 self.coefficients.update(logistic_regression_info.get( \
                     'coefficients', []))
-                if not isinstance(self.coefficients.values()[0][0], list):
+                if not isinstance(list(self.coefficients.values())[0][0], list):
                     old_coefficients = True
                 self.bias = logistic_regression_info.get('bias', True)
                 self.c = logistic_regression_info.get('c')
@@ -166,7 +175,7 @@ class LogisticRegression(ModelFields):
                 missing_tokens = logistic_regression_info.get("missing_tokens")
                 ModelFields.__init__(
                     self, fields,
-                    objective_id=objective_id, terms=True, categories=True,
+                    objective_id=objective_id, categories=True,
                     numerics=True, missing_tokens=missing_tokens)
                 self.field_codings = logistic_regression_info.get( \
                   'field_codings', {})
@@ -180,17 +189,14 @@ class LogisticRegression(ModelFields):
                         del self.field_codings[field_id]
                 if old_coefficients:
                     self.map_coefficients()
-                categories = self.fields[self.objective_id].get( \
-                    "summary", {}).get('categories')
-                if len(self.coefficients.keys()) > len(categories):
+                categories = self.categories[self.objective_id]
+                if len(list(self.coefficients.keys())) > len(categories):
                     self.class_names = [""]
                 else:
                     self.class_names = []
-                self.class_names.extend(sorted([category[0]
-                                                for category in categories]))
+                self.class_names.extend(sorted(categories))
                 # order matters
-                self.objective_categories = [category[0]
-                                             for category in categories]
+                self.objective_categories = categories
             else:
                 raise Exception("The logistic regression isn't finished yet")
         else:
@@ -225,8 +231,7 @@ class LogisticRegression(ModelFields):
 
         if compact:
             return [category['probability'] for category in distribution]
-        else:
-            return distribution
+        return distribution
 
     def predict_operating(self, input_data,
                           operating_point=None):
@@ -311,49 +316,47 @@ class LogisticRegression(ModelFields):
 
         # Checks and cleans input_data leaving the fields used in the model
         unused_fields = []
-        new_data = self.filter_input_data( \
+        norm_input_data = self.filter_input_data( \
             input_data,
             add_unused_fields=full)
         if full:
-            input_data, unused_fields = new_data
-        else:
-            input_data = new_data
+            norm_input_data, unused_fields = norm_input_data
 
         # Strips affixes for numeric values and casts to the final field type
-        cast(input_data, self.fields)
+        cast(norm_input_data, self.fields)
 
         # When operating_point is used, we need the probabilities
         # of all possible classes to decide, so se use
         # the `predict_probability` method
         if operating_point:
             return self.predict_operating( \
-                input_data, operating_point=operating_point)
+                norm_input_data, operating_point=operating_point)
         if operating_kind:
             return self.predict_operating_kind( \
-                input_data, operating_kind=operating_kind)
+                norm_input_data, operating_kind=operating_kind)
 
         # In case that missing_numerics is False, checks that all numeric
         # fields are present in input data.
-        if not self.missing_numerics:
-            check_no_missing_numerics(input_data, self.model_fields,
+        if not self.missing_numerics and self.default_numeric_value is None:
+            check_no_missing_numerics(norm_input_data, self.model_fields,
                                       self.weight_field)
 
         if self.balance_fields:
-            balance_input(input_data, self.fields)
+            balance_input(norm_input_data, self.fields)
 
         # Computes text and categorical field expansion
-        unique_terms = self.get_unique_terms(input_data)
+        unique_terms = self.get_unique_terms(norm_input_data)
 
         probabilities = {}
         total = 0
         # Computes the contributions for each category
         for category in self.coefficients:
             probability = self.category_probability( \
-                input_data, unique_terms, category)
+                norm_input_data, unique_terms, category)
             try:
                 order = self.categories[self.objective_id].index(category)
             except ValueError:
-                if category == u'':
+                if category == '':
                     order = len(self.categories[self.objective_id])
             probabilities[category] = {"category": category,
                                        "probability": probability,
@@ -366,7 +369,7 @@ class LogisticRegression(ModelFields):
                 probabilities[category]["probability"], PRECISION)
 
         # Chooses the most probable category as prediction
-        predictions = sorted(probabilities.items(),
+        predictions = sorted(list(probabilities.items()),
                              key=lambda x: (x[1]["probability"],
                                             - x[1]["order"]), reverse=True)
         for prediction, probability in predictions:
@@ -414,7 +417,7 @@ class LogisticRegression(ModelFields):
                             index = self.items[field_id].index(term)
                         elif field_id in self.categories and ( \
                                 not field_id in self.field_codings or \
-                                self.field_codings[field_id].keys()[0] == \
+                                list(self.field_codings[field_id].keys())[0] == \
                                 "dummy"):
                             index = self.categories[field_id].index(term)
                         elif field_id in self.categories:
@@ -422,7 +425,7 @@ class LogisticRegression(ModelFields):
                             index = self.categories[field_id].index(term)
                             coeff_index = 0
                             for contribution in \
-                                    self.field_codings[field_id].values()[0]:
+                                    list(self.field_codings[field_id].values())[0]:
                                 probability += \
                                     coefficients[coeff_index] * \
                                     contribution[index] * occurrences
@@ -457,17 +460,13 @@ class LogisticRegression(ModelFields):
                     field_id != self.objective_id and \
                     field_id not in unique_terms:
                 if field_id not in self.field_codings or \
-                        self.field_codings[field_id].keys()[0] == "dummy":
+                        list(self.field_codings[field_id].keys())[0] == "dummy":
                     probability += coefficients[ \
                         len(self.categories[field_id])]
                 else:
-                    """ codings are given as arrays of coefficients. The
-                    last one is for missings and the previous ones are
-                    one per category as found in summary
-                    """
                     coeff_index = 0
                     for contribution in \
-                            self.field_codings[field_id].values()[0]:
+                            list(self.field_codings[field_id].values())[0]:
                         probability += coefficients[coeff_index] * \
                             contribution[-1]
                         coeff_index += 1
@@ -506,7 +505,7 @@ class LogisticRegression(ModelFields):
         shift = 0
         for field_id in field_ids:
             optype = self.fields[field_id]['optype']
-            if optype in EXPANSION_ATTRIBUTES.keys():
+            if optype in list(EXPANSION_ATTRIBUTES.keys()):
                 # text and items fields have one coefficient per
                 # text plus a missing terms coefficient plus a bias
                 # coefficient
@@ -514,13 +513,13 @@ class LogisticRegression(ModelFields):
                 # field coding.
                 if optype != 'categorical' or \
                         not field_id in self.field_codings or \
-                        self.field_codings[field_id].keys()[0] == "dummy":
-                    length = len(self.fields[field_id]['summary'][ \
-                        EXPANSION_ATTRIBUTES[optype]])
+                        list(self.field_codings[field_id].keys())[0] == "dummy":
+                    length = len(getattr(
+                        self, EXPANSION_ATTRIBUTES[optype])[field_id])
                     # missing coefficient
                     length += 1
                 else:
-                    length = len(self.field_codings[field_id].values()[0])
+                    length = len(list(self.field_codings[field_id].values())[0])
             else:
                 # numeric fields have one coefficient and an additional one
                 # if self.missing_numerics is True
