@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
 #
-# Copyright 2013-2020 BigML
+# Copyright 2013-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -26,19 +25,24 @@ is used for local predictions.
 import logging
 import re
 
-from bigml.util import invert_dictionary, DEFAULT_LOCALE
-from bigml.constants import DEFAULT_MISSING_TOKENS, FIELDS_PARENT
-from bigml.resourcehandler import get_resource_type, resource_is_ready
-from bigml.predicate import TM_FULL_TERM, TM_ALL
 from bigml_chronos import chronos
 
+from bigml.util import invert_dictionary, dump, dumps, DEFAULT_LOCALE
+from bigml.constants import DEFAULT_MISSING_TOKENS, FIELDS_PARENT, \
+    ENSEMBLE_PATH
+from bigml.api_handlers.resourcehandler import get_resource_type
+from bigml.predicate import TM_FULL_TERM, TM_ALL
+
+
 
 LOGGER = logging.getLogger('BigML')
 
 DATE_FNS = {
-    "day-of-month": lambda (x): x.day,
-    "day-of-week": lambda (x): x.weekday() + 1,
-    "millisecond": lambda(x): x.microsecond / 1000}
+    "day-of-month": lambda x: x.day,
+    "day-of-week": lambda x: x.weekday() + 1,
+    "millisecond": lambda x: x.microsecond / 1000}
+
+NUMERIC = "numeric"
 
 
 def parse_terms(text, case_sensitive=True):
@@ -47,7 +51,7 @@ def parse_terms(text, case_sensitive=True):
     """
     if text is None:
         return []
-    expression = ur'(\b|_)([^\b_\s]+?)(\b|_)'
+    expression = r'(\b|_)([^\b_\s]+?)(\b|_)'
     pattern = re.compile(expression)
     return [match[1] if case_sensitive else match[1].lower()
             for match in re.findall(pattern, text)]
@@ -75,8 +79,8 @@ def check_model_fields(model):
         input_fields = model.get("input_fields")
         # models only need model_fields to work. The rest of resources will
         # need all fields to work
-        model_fields = model.get(inner_key, {}).get( \
-            'model_fields', {}).keys()
+        model_fields = list(model.get(inner_key, {}).get( \
+            'model_fields', {}).keys())
         # fusions don't have input fields
         if input_fields is None and inner_key != "fusion":
             return False
@@ -92,34 +96,24 @@ def check_model_fields(model):
         else:
             if fields is None:
                 return False
-            return all([field_id in fields.keys() \
+            return all([field_id in list(fields.keys()) \
                 for field_id in model_fields])
     return False
 
 
-def check_model_structure(model, inner_key="model"):
+def check_model_structure(model, inner_key=None):
     """Checks the model structure to see if it contains all the
     main expected keys
 
     """
+    if inner_key is None:
+        inner_key = FIELDS_PARENT.get(get_resource_type(model), 'model')
     return (isinstance(model, dict) and 'resource' in model and
             model['resource'] is not None and
-            ('object' in model and inner_key in model['object'] or
+            (('object' in model and inner_key in model['object']) or
              inner_key in model))
 
 
-def lacks_info(model, inner_key="model"):
-    """Whether the information in `model` is not enough to use it locally
-
-    """
-    try:
-        return not (resource_is_ready(model) and \
-            check_model_structure(model, inner_key) and \
-            check_model_fields(model))
-    except Exception:
-        return True
-
-
 def get_unique_terms(terms, term_forms, tag_cloud):
     """Extracts the unique terms that occur in one of the alternative forms in
        term_forms or in the tag cloud.
@@ -127,7 +121,7 @@ def get_unique_terms(terms, term_forms, tag_cloud):
     """
 
     extend_forms = {}
-    for term, forms in term_forms.items():
+    for term, forms in list(term_forms.items()):
         for form in forms:
             extend_forms[form] = term
         extend_forms[term] = term
@@ -142,7 +136,7 @@ def get_unique_terms(terms, term_forms, tag_cloud):
             if term not in terms_set:
                 terms_set[term] = 0
             terms_set[term] += 1
-    return terms_set.items()
+    return list(terms_set.items())
 
 
 def get_datetime_subfields(fields):
@@ -151,12 +145,11 @@ def get_datetime_subfields(fields):
 
     """
     subfields = {}
-    for fid, finfo in fields.items():
+    for fid, finfo in list(fields.items()):
         if finfo.get('parent_optype', False) == 'datetime':
             parent_id = finfo["parent_ids"][0]
-            parent_name = fields[parent_id]["name"]
             subfield = {fid: finfo["datatype"]}
-            if parent_id in subfields.keys():
+            if parent_id in list(subfields.keys()):
                 subfields[parent_id].update(subfield)
             else:
                 subfields[parent_id] = subfield
@@ -173,7 +166,7 @@ def expand_date(date, subfields, timeformats):
         parsed_date = chronos.parse(date, format_names=timeformats)
     except ValueError:
         return {}
-    for fid, ftype in subfields.items():
+    for fid, ftype in list(subfields.items()):
         date_fn = DATE_FNS.get(ftype)
         if date_fn is not None:
             expanded.update({fid: date_fn(parsed_date)})
@@ -188,7 +181,7 @@ def get_datetime_formats(fields):
 
     """
     timeformats = {}
-    for f_id, finfo in fields.items():
+    for f_id, finfo in list(fields.items()):
         if finfo.get('optype', False) == 'datetime':
             timeformats[f_id] = finfo.get('time_formats', {})
     return timeformats
@@ -199,20 +192,20 @@ def add_expanded_dates(input_data, datetime_fields):
     provided by the user (only if the user didn't specify it)
 
     """
-    for index, value in datetime_fields.items():
+    for index, value in list(datetime_fields.items()):
         if index not in input_data:
             input_data[index] = value
     return input_data
 
 
-class ModelFields(object):
+class ModelFields():
     """ A lightweight wrapper of the field information in the model, cluster
     or anomaly objects
 
     """
 
     def __init__(self, fields, objective_id=None, data_locale=None,
-                 missing_tokens=None, terms=False, categories=False,
+                 missing_tokens=None, categories=False,
                  numerics=False):
         if isinstance(fields, dict):
             try:
@@ -223,18 +216,15 @@ class ModelFields(object):
                 self.fields.update(fields)
                 if not (hasattr(self, "input_fields") and self.input_fields):
                     self.input_fields = [field_id for field_id, field in \
-                        sorted( \
-                        [(field_id, field) for field_id,
-                         field in self.fields.items()],
-                        key=lambda(x): x[1].get("column_number")) \
+                        sorted(list(self.fields.items()),
+                               key=lambda x: x[1].get("column_number")) \
                         if not self.objective_id or \
                         field_id != self.objective_id]
                 self.model_fields = {}
                 self.datetime_parents = []
-                self.model_fields.update(
-                    dict([(field_id, field) for field_id, field in \
-                    self.fields.items() if field_id in self.input_fields and \
-                    self.fields[field_id].get("preferred", True)]))
+                self.model_fields.update({field_id: field for field_id, field \
+                    in self.fields.items() if field_id in self.input_fields \
+                    and self.fields[field_id].get("preferred", True)})
                 # if any of the model fields is a generated datetime field
                 # we need to add the parent datetime field
                 self.model_fields = self.add_datetime_parents()
@@ -244,18 +234,27 @@ class ModelFields(object):
                     self.data_locale = DEFAULT_LOCALE
                 if self.missing_tokens is None:
                     self.missing_tokens = DEFAULT_MISSING_TOKENS
-                if terms:
-                    # adding text and items information to handle terms
-                    # expansion
-                    self.term_forms = {}
-                    self.tag_clouds = {}
-                    self.term_analysis = {}
-                    self.items = {}
-                    self.item_analysis = {}
+                # adding text and items information to handle terms
+                # expansion
+                self.term_forms = []
+                self.tag_clouds = {}
+                self.term_analysis = {}
+                self.items = {}
+                self.item_analysis = {}
                 if categories:
                     self.categories = {}
-                if terms or categories or numerics:
-                    self.add_terms(categories, numerics)
+                self.add_terms(categories, numerics)
+
+                if self.objective_id is not None and \
+                        hasattr(self, "resource_id") and self.resource_id and \
+                        get_resource_type(self.resource_id) != ENSEMBLE_PATH:
+                    # Only for models. Ensembles need their own logic
+                    self.regression = \
+                        (not hasattr(self, "boosting") or not self.boosting) \
+                        and self.fields[self.objective_id][ \
+                        'optype'] == NUMERIC \
+                        or (hasattr(self, "boosting") and self.boosting and \
+                        self.boosting.get("objective_class") is None)
 
             except KeyError:
                 raise Exception("Wrong field structure.")
@@ -264,27 +263,31 @@ class ModelFields(object):
         """Adds the terms information of text and items fields
 
         """
-        for field_id, field in self.fields.items():
-            if field['optype'] == 'text':
-                self.term_forms[field_id] = {}
-                self.term_forms[field_id].update(
-                    field['summary']['term_forms'])
+        for field_id, field in list(self.fields.items()):
+            if field['optype'] == 'text' and \
+                    self.fields[field_id]['summary'].get('tag_cloud'):
+                self.term_forms.append(field_id)
                 self.tag_clouds[field_id] = []
                 self.tag_clouds[field_id] = [tag for [tag, _] in field[
                     'summary']['tag_cloud']]
+                del self.fields[field_id]["summary"]["tag_cloud"]
                 self.term_analysis[field_id] = {}
                 self.term_analysis[field_id].update(
                     field['term_analysis'])
-            if field['optype'] == 'items':
+            if field['optype'] == 'items' and \
+                    self.fields[field_id]["summary"].get("items"):
                 self.items[field_id] = []
                 self.items[field_id] = [item for item, _ in \
                     field['summary']['items']]
+                del self.fields[field_id]["summary"]["items"]
                 self.item_analysis[field_id] = {}
                 self.item_analysis[field_id].update(
                     field['item_analysis'])
-            if categories and field['optype'] == 'categorical':
+            if categories and field['optype'] == 'categorical' and \
+                    self.fields[field_id]["summary"]["categories"]:
                 self.categories[field_id] = [category for \
                     [category, _] in field['summary']['categories']]
+                del self.fields[field_id]["summary"]["categories"]
             if field['optype'] == 'datetime' and \
                     hasattr(self, "coeff_ids"):
                 self.coeff_id = [coeff_id for coeff_id in self.coeff_ids \
@@ -300,7 +303,7 @@ class ModelFields(object):
            transformation is applied to ensure unicity.
 
         """
-        unique_names = set([fields[key]['name'] for key in fields])
+        unique_names = {fields[key]['name'] for key in fields}
         if len(unique_names) < len(fields):
             self.transform_repeated_names(fields)
 
@@ -332,8 +335,8 @@ class ModelFields(object):
         """Transforms to unicode and cleans missing tokens
 
         """
-        if isinstance(value, basestring) and not isinstance(value, unicode):
-            value = unicode(value, "utf-8")
+        if isinstance(value, str) and not isinstance(value, str):
+            value = str(value, "utf-8")
         return None if value in self.missing_tokens else value
 
     def expand_datetime_fields(self, input_data):
@@ -344,7 +347,7 @@ class ModelFields(object):
         expanded = {}
         timeformats = get_datetime_formats(self.fields)
         subfields = get_datetime_subfields(self.fields)
-        for f_id, value in input_data.items():
+        for f_id, value in list(input_data.items()):
             if f_id in subfields:
                 formats = timeformats.get(f_id, [])
                 expanded.update(expand_date(value, subfields[f_id], formats))
@@ -355,7 +358,7 @@ class ModelFields(object):
         datetime fields used in the model
         """
         subfields = get_datetime_subfields(self.fields)
-        for f_id in subfields.keys():
+        for f_id in list(subfields.keys()):
             self.model_fields[f_id] = self.fields[f_id]
             self.datetime_parents.append(f_id)
         return self.model_fields
@@ -369,6 +372,23 @@ class ModelFields(object):
                 del input_data[f_id]
         return input_data
 
+    def fill_numeric_defaults(self, input_data):
+        """Fills the value set as default for numeric missing fields if user
+        created the model with the default_numeric_value option
+
+        """
+        if hasattr(self, "default_numeric_value") and \
+                self.default_numeric_value is not None:
+            for key in self.fields:
+                if key in self.model_fields and \
+                        (self.objective_id is None or \
+                         key != self.objective_id) and  \
+                        self.fields[key]["optype"] == NUMERIC and \
+                        input_data.get(key) is None:
+                    input_data[key] = self.fields[key]["summary"].get( \
+                        self.default_numeric_value, 0)
+        return input_data
+
     def filter_input_data(self, input_data,
                           add_unused_fields=False):
         """Filters the keys given in input_data checking against model fields.
@@ -382,11 +402,11 @@ class ModelFields(object):
         tmp_input.update(input_data)
         if isinstance(tmp_input, dict):
             # remove all missing values
-            for key, value in tmp_input.items():
+            for key, value in list(tmp_input.items()):
                 value = self.normalize(value)
                 if value is None:
                     del tmp_input[key]
-            for key, value in tmp_input.items():
+            for key, value in list(tmp_input.items()):
                 if key not in self.fields:
                     key = self.inverted_fields.get(key, key)
                 # only the fields that are listed in input_fields and appear
@@ -400,13 +420,14 @@ class ModelFields(object):
             datetime_fields = self.expand_datetime_fields(new_input)
             new_input = add_expanded_dates(new_input, datetime_fields)
             new_input = self.remove_parent_datetimes(new_input)
+            # we fill the input with the chosen default, if selected
+            new_input = self.fill_numeric_defaults(new_input)
             result = (new_input, unused_fields) if add_unused_fields else \
                 new_input
             return result
-        else:
-            LOGGER.error("Failed to read input data in the expected"
-                         " {field:value} format.")
-            return ({}, []) if add_unused_fields else {}
+        LOGGER.error("Failed to read input data in the expected"
+                     " {field:value} format.")
+        return ({}, []) if add_unused_fields else {}
 
     def get_unique_terms(self, input_data):
         """Parses the input data to find the list of unique terms in the
@@ -417,7 +438,7 @@ class ModelFields(object):
         for field_id in self.term_forms:
             if field_id in input_data:
                 input_data_field = input_data.get(field_id, '')
-                if isinstance(input_data_field, basestring):
+                if isinstance(input_data_field, str):
                     case_sensitive = self.term_analysis[field_id].get(
                         'case_sensitive', True)
                     token_mode = self.term_analysis[field_id].get(
@@ -438,7 +459,7 @@ class ModelFields(object):
                                                        terms[0] != full_term)):
                         terms.append(full_term)
                     unique_terms[field_id] = get_unique_terms(
-                        terms, self.term_forms[field_id],
+                        terms, self.fields[field_id]["summary"]["term_forms"],
                         self.tag_clouds.get(field_id, []))
                 else:
                     unique_terms[field_id] = [(input_data_field, 1)]
@@ -447,14 +468,14 @@ class ModelFields(object):
         for field_id in self.item_analysis:
             if field_id in input_data:
                 input_data_field = input_data.get(field_id, '')
-                if isinstance(input_data_field, basestring):
+                if isinstance(input_data_field, str):
                     # parsing the items in input_data
                     separator = self.item_analysis[field_id].get(
                         'separator', ' ')
                     regexp = self.item_analysis[field_id].get(
                         'separator_regexp')
                     if regexp is None:
-                        regexp = ur'%s' % re.escape(separator)
+                        regexp = r'%s' % re.escape(separator)
                     terms = parse_items(input_data_field, regexp)
                     unique_terms[field_id] = get_unique_terms(
                         terms, {},
@@ -470,3 +491,18 @@ class ModelFields(object):
                     unique_terms[field_id] = [(input_data_field, 1)]
                     del input_data[field_id]
         return unique_terms
+
+    def dump(self, output=None, cache_set=None):
+        """Uses msgpack to serialize the resource object
+        If cache_set is filled with a cache set method, the method is called
+
+        """
+        self_vars = vars(self)
+        dump(self_vars, output=output, cache_set=cache_set)
+
+    def dumps(self):
+        """Uses msgpack to serialize the resource object to a string
+
+        """
+        self_vars = vars(self)
+        dumps(self_vars)
