@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
 #
-# Copyright 2012-2020 BigML
+# Copyright 2012-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -39,8 +38,10 @@ model.predict({"petal length": 3, "petal width": 1})
 import logging
 import ast
 
+from functools import partial
 
-from bigml.model import Model, cast_prediction
+from bigml.exceptions import NoRootDecisionTree
+from bigml.model import Model, cast_prediction, to_prediction
 from bigml.model import LAST_PREDICTION
 from bigml.util import get_predictions_file_name
 from bigml.multivote import MultiVote
@@ -52,7 +53,7 @@ from bigml.io import UnicodeWriter, UnicodeReader
 LOGGER = logging.getLogger('BigML')
 
 
-def read_votes(votes_files, to_prediction, data_locale=None):
+def read_votes(votes_files, to_prediction_fn, data_locale=None):
     """Reads the votes found in the votes' files.
 
        Returns a list of MultiVote objects containing the list of predictions.
@@ -70,12 +71,11 @@ def read_votes(votes_files, to_prediction, data_locale=None):
        used in numeric formatting.
     """
     votes = []
-    for order in range(0, len(votes_files)):
-        votes_file = votes_files[order]
+    for order, votes_file in enumerate(votes_files):
         index = 0
         with UnicodeReader(votes_file) as rdr:
             for row in rdr:
-                prediction = to_prediction(row[0], data_locale=data_locale)
+                prediction = to_prediction_fn(row[0], data_locale=data_locale)
                 if index > (len(votes) - 1):
                     votes.append(MultiVote([]))
                 distribution = None
@@ -94,7 +94,7 @@ def read_votes(votes_files, to_prediction, data_locale=None):
     return votes
 
 
-class MultiModel(object):
+class MultiModel():
     """A multiple local model.
 
     Uses a number of BigML remote models to build a local version that can be
@@ -102,7 +102,9 @@ class MultiModel(object):
 
     """
 
-    def __init__(self, models, api=None, fields=None, class_names=None):
+    def __init__(self, models, api=None, fields=None, class_names=None,
+                 cache_get=None):
+
         self.models = []
         self.class_names = class_names
 
@@ -111,9 +113,18 @@ class MultiModel(object):
                 self.models = models
             else:
                 for model in models:
-                    self.models.append(Model(model, api=api, fields=fields))
+                    # some models have no root info and should not be added
+                    try:
+                        self.models.append(Model(model, api=api, fields=fields,
+                                                 cache_get=cache_get))
+                    except NoRootDecisionTree:
+                        pass
         else:
-            self.models.append(Model(models, api=api, fields=fields))
+            try:
+                self.models.append(Model(models, api=api, fields=fields,
+                                         cache_get=cache_get))
+            except NoRootDecisionTree:
+                pass
 
     def list_models(self):
         """Lists all the model/ids that compound the multi model.
@@ -277,11 +288,11 @@ class MultiModel(object):
                 out.open_writer()
             for index, input_data in enumerate(input_data_list):
                 if add_headers:
-                    input_data = dict(zip(headers, input_data))
+                    input_data = dict(list(zip(headers, input_data)))
                 prediction = model.predict(input_data,
                                            missing_strategy=missing_strategy,
                                            full=True)
-                if model.tree.regression:
+                if model.regression:
                     # if median is to be used, we just replace the prediction
                     if use_median:
                         prediction["prediction"] = prediction["median"]
@@ -314,4 +325,5 @@ class MultiModel(object):
                     model.resource_id,
                     predictions_file_path))
         return read_votes(
-            votes_files, self.models[0].to_prediction, data_locale=data_locale)
+            votes_files, partial(to_prediction, self.models[0]),
+            data_locale=data_locale)
