@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
+
 #
-# Copyright 2012-2020 BigML
+# Copyright 2012-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -20,7 +20,7 @@ import json
 import os
 from nose.tools import eq_, assert_less
 from datetime import datetime
-from world import world, res_filename, logged_wait
+from .world import world, res_filename, logged_wait
 
 from bigml.api import HTTP_OK
 from bigml.api import HTTP_CREATED
@@ -35,19 +35,20 @@ from bigml.linear import LinearRegression
 from bigml.deepnet import Deepnet
 from bigml.fusion import Fusion
 
-import read_model_steps as read
+from . import read_model_steps as read
 
 NO_MISSING_SPLITS = {'missing_splits': False}
 
 #@step(r'I create a model$')
-def i_create_a_model(step):
-    dataset = world.dataset.get('resource')
-    resource = world.api.create_model(dataset, args=NO_MISSING_SPLITS)
-    world.status = resource['code']
-    eq_(world.status, HTTP_CREATED)
-    world.location = resource['location']
-    world.model = resource['object']
-    world.models.append(resource['resource'])
+def i_create_a_model(step, shared=None):
+    if shared is None or step.shared.get(shared, {}).get("model") is None:
+        dataset = world.dataset.get('resource')
+        resource = world.api.create_model(dataset, args=NO_MISSING_SPLITS)
+        world.status = resource['code']
+        eq_(world.status, HTTP_CREATED)
+        world.location = resource['location']
+        world.model = resource['object']
+        world.models.append(resource['resource'])
 
 #@step(r'I export the model$')
 def i_export_model(step, filename):
@@ -100,8 +101,14 @@ def wait_until_model_status_code_is(step, code1, code2, secs):
     eq_(status['code'], int(code1))
 
 #@step(r'I wait until the model is ready less than (\d+)')
-def the_model_is_finished_in_less_than(step, secs):
-    wait_until_model_status_code_is(step, FINISHED, FAULTY, secs)
+def the_model_is_finished_in_less_than(step, secs, shared=None):
+    if shared is None or step.shared.get(shared, {}).get("model") is None:
+        wait_until_model_status_code_is(step, FINISHED, FAULTY, secs)
+        if shared is not None:
+            step.shared[shared]["model"] = world.model
+    else:
+        world.model = step.shared[shared]["model"]
+
 
 #@step(r'I create a model with "(.*)"')
 def i_create_a_model_with(step, data="{}"):
@@ -131,7 +138,7 @@ def make_the_model_public(step):
                                       {'private': False, 'white_box': True})
     world.status = resource['code']
     if world.status != HTTP_ACCEPTED:
-        print "unexpected status: %s" % world.status
+        print("unexpected status: %s" % world.status)
     eq_(world.status, HTTP_ACCEPTED)
     world.location = resource['location']
     world.model = resource['object']
@@ -169,7 +176,7 @@ def model_from_shared_key(step):
 
 #@step(r'"(.*)" field\'s name is changed to "(.*)"')
 def field_name_to_new_name(step, field_id, new_name):
-    eq_(world.local_model.tree.fields[field_id]['name'], new_name)
+    eq_(world.local_model.fields[field_id]['name'], new_name)
 
 #@step(r'I create a model associated to centroid "(.*)"')
 def i_create_a_model_from_cluster(step, centroid_id):
@@ -202,13 +209,14 @@ def i_create_a_logistic_model(step):
 
 
 #@step(r'I create a logistic regression model with objective "(.*?)" and parms "(.*)"$')
-def i_create_a_logistic_model_with_objective_and_parms(step, objective, parms=None):
+def i_create_a_logistic_model_with_objective_and_parms(step, objective=None, parms=None):
     dataset = world.dataset.get('resource')
     if parms is None:
         parms = {}
     else:
         parms = json.loads(parms)
-    parms.update({"objective_field": objective})
+    if objective is not None:
+        parms.update({"objective_field": objective})
     resource = world.api.create_logistic_regression( \
         dataset, parms)
     world.status = resource['code']
@@ -260,13 +268,14 @@ def i_create_a_no_suggest_deepnet(step):
     world.deepnets.append(resource['resource'])
 
 #@step(r'I create a deepnet model with objective "(.*?)" and parms "(.*)"$')
-def i_create_a_deepnet_with_objective_and_params(step, objective, parms=None):
+def i_create_a_deepnet_with_objective_and_params(step, objective=None, parms=None):
     dataset = world.dataset.get('resource')
     if parms is None:
         parms = {}
     else:
         parms = json.loads(parms)
-    parms.update({"objective_field": objective})
+    if objective is not None:
+        parms.update({"objective_field": objective})
     resource = world.api.create_deepnet(dataset, parms)
     world.status = resource['code']
     eq_(world.status, HTTP_CREATED)
@@ -304,6 +313,12 @@ def i_check_model_stored(step, filename, pmml):
             (world.model["resource"].index("/") + 1):]
         assert(content.index(model_id) > -1)
 
+#@step(r'I read model from file "(.*)"$')
+def i_read_model_file(step, filename):
+    with open(res_filename(filename)) as file_handler:
+        content = file_handler.read()
+        world.model = json.loads(content)
+
 #@step(r'I create an optiml$')
 def i_create_an_optiml(step):
     dataset = world.dataset.get('resource')
@@ -365,7 +380,8 @@ def i_check_optiml_name(step, name):
 
 #@step(r'I create a fusion$')
 def i_create_a_fusion(step):
-    resource = world.api.create_fusion(world.list_of_models)
+    resource = world.api.create_fusion(world.list_of_models,
+                                       {"project": world.project_id})
     world.status = resource['code']
     eq_(world.status, HTTP_CREATED)
     world.location = resource['location']
@@ -376,7 +392,7 @@ def i_create_a_fusion(step):
 #@step(r'I create a fusion with weights$')
 def i_create_a_fusion_with_weights(step, weights=None):
     if weights is None:
-        weights = range(1, len(world.list_of_models))
+        weights = list(range(1, len(world.list_of_models)))
     else:
         weights = json.loads(weights)
     models = []
@@ -385,7 +401,8 @@ def i_create_a_fusion_with_weights(step, weights=None):
             models.append({"id": model["resource"], "weight": weights[index]})
     except IndexError:
         pass
-    resource = world.api.create_fusion(models)
+    resource = world.api.create_fusion(models,
+                                       {"project": world.project_id})
     world.status = resource['code']
     eq_(world.status, HTTP_CREATED)
     world.location = resource['location']
@@ -399,7 +416,7 @@ def i_create_a_fusion_with_objective_and_params(step, objective, parms=None):
         parms = {}
     else:
         parms = json.loads(parms)
-    parms.update({"objective_field": objective})
+    parms.update({"objective_field": objective, "project": world.project_id})
     resource = world.api.create_fusion(models, parms)
     world.status = resource['code']
     eq_(world.status, HTTP_CREATED)
@@ -534,3 +551,30 @@ def i_create_local_linear_regression_from_file(step, export_file):
 def check_linear_regression_id_local_id(step):
     eq_(world.local_linear_regression.resource_id,
         world.linear_regression["resource"])
+
+def local_logistic_prediction_is(step, input_data, prediction):
+    eq_(world.local_logistic.predict(input_data), prediction)
+
+def local_linear_prediction_is(step, input_data, prediction):
+    eq_(world.local_linear_regression.predict(input_data), prediction)
+
+def local_deepnet_prediction_is(step, input_data, prediction):
+    eq_(world.local_deepnet.predict(input_data), prediction)
+
+def local_ensemble_prediction_is(step, input_data, prediction):
+    eq_(world.local_ensemble.predict(input_data), prediction)
+
+def local_model_prediction_is(step, input_data, prediction):
+    eq_(world.local_model.predict(input_data), prediction)
+
+def local_cluster_prediction_is(step, input_data, prediction):
+    eq_(world.local_cluster.centroid(input_data), prediction)
+
+def local_anomaly_prediction_is(step, input_data, prediction):
+    eq_(world.local_anomaly.anomaly_score(input_data), prediction)
+
+def local_association_prediction_is(step, input_data, prediction):
+    eq_(world.local_association.association_set(input_data), prediction)
+
+def local_time_series_prediction_is(step, input_data, prediction):
+    eq_(world.local_time_series.centroid(input_data), prediction)
