@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
+
 #
-# Copyright 2014-2020 BigML
+# Copyright 2014-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -15,7 +15,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from world import world
+from .world import world
 from nose.tools import ok_
 
 #@step(r'I store the dataset id in a list')
