@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
+
 #
-# Copyright 2015-2020 BigML
+# Copyright 2015-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -19,7 +19,7 @@ import time
 import json
 import os
 from datetime import datetime
-from world import world, logged_wait, res_filename
+from .world import world, logged_wait, res_filename
 from nose.tools import eq_, assert_less
 
 from bigml.api import HTTP_CREATED
@@ -29,7 +29,7 @@ from bigml.api import FAULTY
 from bigml.api import get_status
 from bigml.util import is_url
 
-from read_script_steps import i_get_the_script
+from .read_script_steps import i_get_the_script
 
 
 #@step(r'the script code is "(.*)" and the value of "(.*)" is "(.*)"')
@@ -42,7 +42,8 @@ def the_script_code_and_attributes(step, source_code, param, param_value):
 
 #@step(r'I create a whizzml script from a excerpt of code "(.*)"$')
 def i_create_a_script(step, source_code):
-    resource = world.api.create_script(source_code)
+    resource = world.api.create_script(source_code,
+                                       {"project": world.project_id})
     world.status = resource['code']
     eq_(world.status, HTTP_CREATED)
     world.location = resource['location']
@@ -54,7 +55,8 @@ def i_create_a_script(step, source_code):
 def i_create_a_script_from_file_or_url(step, source_code):
     if not is_url(source_code):
         source_code = res_filename(source_code)
-    resource = world.api.create_script(source_code)
+    resource = world.api.create_script(source_code,
+                                       {"project": world.project_id})
     world.status = resource['code']
     eq_(world.status, HTTP_CREATED)
     world.location = resource['location']
