@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
 #
-# Copyright 2014-2020 BigML
+# Copyright 2014-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -17,7 +16,7 @@
 
 import time
 from datetime import datetime, timedelta
-from world import world
+from .world import world
 from nose.tools import eq_, assert_less
 from bigml.api import HTTP_NO_CONTENT, HTTP_OK, HTTP_NOT_FOUND
 
