@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
 #
-# Copyright 2015-2020 BigML
+# Copyright 2015-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -16,7 +15,7 @@
 # under the License.
 
 from nose.tools import eq_
-from world import world
+from .world import world
 from bigml.api import HTTP_OK
 
 #@step(r'I get the test "(.*)"')
