@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
 #
-# Copyright 2015-2020 BigML
+# Copyright 2015-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -19,26 +18,31 @@
 """ Testing local model information output methods
 
 """
-from world import world, setup_module, teardown_module
-import create_source_steps as source_create
-import create_dataset_steps as dataset_create
-import create_model_steps as model_create
-import compare_predictions_steps as prediction_compare
-import inspect_model_steps as inspect_model
+
+
+from .world import world, setup_module, teardown_module
+from . import create_source_steps as source_create
+from . import create_dataset_steps as dataset_create
+from . import create_model_steps as model_create
+from . import compare_predictions_steps as prediction_compare
+from . import inspect_model_steps as inspect_model
 
 class TestLocalModelOutputs(object):
 
+    def __init__(self):
+        self.shared = {} # stores shared objects references
+
     def setup(self):
         """
             Debug information
         """
-        print "\n-------------------\nTests in: %s\n" % __name__
+        print("\n-------------------\nTests in: %s\n" % __name__)
 
     def teardown(self):
         """
             Debug information
         """
-        print "\nEnd of tests in: %s\n-------------------\n" % __name__
+        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
 
     def test_scenario1(self):
         """
@@ -64,7 +68,7 @@ class TestLocalModelOutputs(object):
                 | data/tiny_kdd.csv      | 20      | 20     | 30     | data/model/if_then_rules_tiny_kdd.txt          |
 
         """
-        print self.test_scenario1.__doc__
+        print(self.test_scenario1.__doc__)
         examples = [
             ['data/iris.csv', '30', '30', '30', 'data/model/if_then_rules_iris.txt'],
             ['data/iris_sp_chars.csv', '30', '30', '30', 'data/model/if_then_rules_iris_sp_chars.txt'],
@@ -74,13 +78,16 @@ class TestLocalModelOutputs(object):
             ['data/iris_missing2.csv', '30', '30', '30', 'data/model/if_then_rules_iris_missing2.txt'],
             ['data/tiny_kdd.csv', '30', '30', '30', 'data/model/if_then_rules_tiny_kdd.txt']]
         for example in examples:
-            print "\nTesting with:\n", example
-            source_create.i_upload_a_file(self, example[0])
-            source_create.the_source_is_finished(self, example[1])
-            dataset_create.i_create_a_dataset(self)
-            dataset_create.the_dataset_is_finished_in_less_than(self, example[2])
-            model_create.i_create_a_model(self)
-            model_create.the_model_is_finished_in_less_than(self, example[3])
+            print("\nTesting with:\n", example)
+            source_create.i_upload_a_file(self, example[0], shared=example[0])
+            source_create.the_source_is_finished(self, example[1],
+                shared=example[0])
+            dataset_create.i_create_a_dataset(self, shared=example[0])
+            dataset_create.the_dataset_is_finished_in_less_than(self,
+                example[2], shared=example[0])
+            model_create.i_create_a_model(self, shared=example[0])
+            model_create.the_model_is_finished_in_less_than(self, example[3],
+                shared=example[0])
             prediction_compare.i_create_a_local_model(self)
             inspect_model.i_translate_the_tree_into_IF_THEN_rules(self)
             inspect_model.i_check_if_the_output_is_like_expected_file(self, example[4])
@@ -103,15 +110,16 @@ class TestLocalModelOutputs(object):
                 | data/iris_missing2.csv | 10      | 10     | 10     | data/model/if_then_rules_iris_missing2_MISSINGS.txt     |
 
         """
-        print self.test_scenario2.__doc__
+        print(self.test_scenario2.__doc__)
         examples = [
             ['data/iris_missing2.csv', '10', '10', '30', 'data/model/if_then_rules_iris_missing2_MISSINGS.txt']]
         for example in examples:
-            print "\nTesting with:\n", example
-            source_create.i_upload_a_file(self, example[0])
-            source_create.the_source_is_finished(self, example[1])
-            dataset_create.i_create_a_dataset(self)
-            dataset_create.the_dataset_is_finished_in_less_than(self, example[2])
+            print("\nTesting with:\n", example)
+            source_create.i_upload_a_file(self, example[0], shared=example[0])
+            source_create.the_source_is_finished(self, example[1], shared=example[0])
+            dataset_create.i_create_a_dataset(self, shared=example[0])
+            dataset_create.the_dataset_is_finished_in_less_than(self,
+                example[2], shared=example[0])
             model_create.i_create_a_model_with_missing_splits(self)
             model_create.the_model_is_finished_in_less_than(self, example[3])
             prediction_compare.i_create_a_local_model(self)
@@ -142,7 +150,7 @@ class TestLocalModelOutputs(object):
                 | data/spam.csv          | 20      | 20     | 30     | {"fields": {"000001": {"optype": "text", "term_analysis": {"case_sensitive": true, "stem_words": true, "use_stopwords": false, "language": "en"}}}} | data/model/if_then_rules_spam_textanalysis_6.txt              |
 
         """
-        print self.test_scenario3.__doc__
+        print(self.test_scenario3.__doc__)
         examples = [
             ['data/spam.csv', '30', '30', '30', '{"fields": {"000001": {"optype": "text", "term_analysis": {"case_sensitive": true, "stem_words": true, "use_stopwords": false, "language": "en"}}}}','data/model/if_then_rules_spam_textanalysis_1.txt'],
             ['data/spam.csv', '30', '30', '30', '{"fields": {"000001": {"optype": "text", "term_analysis": {"case_sensitive": true, "stem_words": true, "use_stopwords": false}}}}', 'data/model/if_then_rules_spam_textanalysis_2.txt'],
@@ -151,12 +159,13 @@ class TestLocalModelOutputs(object):
             ['data/spam.csv', '30', '30', '30', '{"fields": {"000001": {"optype": "text", "term_analysis": {"token_mode": "full_terms_only", "language": "en"}}}}', 'data/model/if_then_rules_spam_textanalysis_5.txt'],
             ['data/spam.csv', '30', '30', '30', '{"fields": {"000001": {"optype": "text", "term_analysis": {"case_sensitive": true, "stem_words": true, "use_stopwords": false, "language": "en"}}}}', 'data/model/if_then_rules_spam_textanalysis_6.txt']]
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             source_create.i_update_source_with(self, example[4])
             dataset_create.i_create_a_dataset(self)
-            dataset_create.the_dataset_is_finished_in_less_than(self, example[2])
+            dataset_create.the_dataset_is_finished_in_less_than(self,
+                example[2])
             model_create.i_create_a_model(self)
             model_create.the_model_is_finished_in_less_than(self, example[3])
             prediction_compare.i_create_a_local_model(self)
@@ -188,7 +197,7 @@ class TestLocalModelOutputs(object):
                 | data/tiny_kdd.csv      | 20      | 20     | 30     | data/model/data_distribution_tiny_kdd.txt          |
 
         """
-        print self.test_scenario4.__doc__
+        print(self.test_scenario4.__doc__)
         examples = [
             ['data/iris.csv', '30', '30', '30', 'data/model/data_distribution_iris.txt'],
             ['data/iris_sp_chars.csv', '30', '30', '30', 'data/model/data_distribution_iris_sp_chars.txt'],
@@ -198,13 +207,15 @@ class TestLocalModelOutputs(object):
             ['data/iris_missing2.csv', '30', '30', '30', 'data/model/data_distribution_iris_missing2.txt'],
             ['data/tiny_kdd.csv', '30', '30', '30', 'data/model/data_distribution_tiny_kdd.txt']]
         for example in examples:
-            print "\nTesting with:\n", example
-            source_create.i_upload_a_file(self, example[0])
-            source_create.the_source_is_finished(self, example[1])
-            dataset_create.i_create_a_dataset(self)
-            dataset_create.the_dataset_is_finished_in_less_than(self, example[2])
-            model_create.i_create_a_model(self)
-            model_create.the_model_is_finished_in_less_than(self, example[3])
+            print("\nTesting with:\n", example)
+            source_create.i_upload_a_file(self, example[0], shared=example[0])
+            source_create.the_source_is_finished(self, example[1], shared=example[0])
+            dataset_create.i_create_a_dataset(self, shared=example[0])
+            dataset_create.the_dataset_is_finished_in_less_than(self,
+                example[2], shared=example[0])
+            model_create.i_create_a_model(self, shared=example[0])
+            model_create.the_model_is_finished_in_less_than(self, example[3],
+                shared=example[0])
             prediction_compare.i_create_a_local_model(self)
             inspect_model.i_check_the_data_distribution(self, example[4])
 
@@ -233,7 +244,7 @@ class TestLocalModelOutputs(object):
                 | data/tiny_kdd.csv      | 20      | 20     | 30     | data/model/predictions_distribution_tiny_kdd.txt          |
 
         """
-        print self.test_scenario5.__doc__
+        print(self.test_scenario5.__doc__)
         examples = [
             ['data/iris.csv', '30', '30', '30', 'data/model/predictions_distribution_iris.txt'],
             ['data/iris_sp_chars.csv', '30', '30', '30', 'data/model/predictions_distribution_iris_sp_chars.txt'],
@@ -243,13 +254,16 @@ class TestLocalModelOutputs(object):
             ['data/iris_missing2.csv', '30', '30', '30', 'data/model/predictions_distribution_iris_missing2.txt'],
             ['data/tiny_kdd.csv', '30', '30', '30', 'data/model/predictions_distribution_tiny_kdd.txt']]
         for example in examples:
-            print "\nTesting with:\n", example
-            source_create.i_upload_a_file(self, example[0])
-            source_create.the_source_is_finished(self, example[1])
-            dataset_create.i_create_a_dataset(self)
-            dataset_create.the_dataset_is_finished_in_less_than(self, example[2])
-            model_create.i_create_a_model(self)
-            model_create.the_model_is_finished_in_less_than(self, example[3])
+            print("\nTesting with:\n", example)
+            source_create.i_upload_a_file(self, example[0], shared=example[0])
+            source_create.the_source_is_finished(self, example[1],
+                shared=example[0])
+            dataset_create.i_create_a_dataset(self, shared=example[0])
+            dataset_create.the_dataset_is_finished_in_less_than(self,
+                example[2], shared=example[0])
+            model_create.i_create_a_model(self, shared=example[0])
+            model_create.the_model_is_finished_in_less_than(self, example[3],
+                shared=example[0])
             prediction_compare.i_create_a_local_model(self)
             inspect_model.i_check_the_predictions_distribution(self, example[4])
 
@@ -278,7 +292,7 @@ class TestLocalModelOutputs(object):
                 | data/tiny_kdd.csv      | 20      | 20     | 30     | data/model/summarize_tiny_kdd.txt          |
 
         """
-        print self.test_scenario6.__doc__
+        print(self.test_scenario6.__doc__)
         examples = [
             ['data/iris.csv', '30', '30', '30', 'data/model/summarize_iris.txt'],
             ['data/iris_sp_chars.csv', '30', '30', '30', 'data/model/summarize_iris_sp_chars.txt'],
@@ -288,12 +302,48 @@ class TestLocalModelOutputs(object):
             ['data/iris_missing2.csv', '30', '30', '30', 'data/model/summarize_iris_missing2.txt'],
             ['data/tiny_kdd.csv', '30', '30', '30', 'data/model/summarize_tiny_kdd.txt']]
         for example in examples:
-            print "\nTesting with:\n", example
-            source_create.i_upload_a_file(self, example[0])
-            source_create.the_source_is_finished(self, example[1])
-            dataset_create.i_create_a_dataset(self)
-            dataset_create.the_dataset_is_finished_in_less_than(self, example[2])
-            model_create.i_create_a_model(self)
-            model_create.the_model_is_finished_in_less_than(self, example[3])
+            print("\nTesting with:\n", example)
+            source_create.i_upload_a_file(self, example[0], shared=example[0])
+            source_create.the_source_is_finished(self, example[1],
+                shared=example[0])
+            dataset_create.i_create_a_dataset(self, shared=example[0])
+            dataset_create.the_dataset_is_finished_in_less_than(self,
+                example[2], shared=example[0])
+            model_create.i_create_a_model(self, shared=example[0])
+            model_create.the_model_is_finished_in_less_than(self, example[3],
+                shared=example[0])
             prediction_compare.i_create_a_local_model(self)
             inspect_model.i_check_the_model_summary_with(self, example[4])
+
+    def test_scenario7(self):
+        """
+            Scenario: Unit tests for output generators:
+                Given I read a model from "<data>" file
+                And I create a local model
+                And I apply a generator function "<function>" with args "<args>"
+                Then I check the result with "<expected_file>" file
+        """
+
+        print(self.test_scenario7.__doc__)
+
+        examples = [
+            ['data/model/iris.json', 'data/model/distribution_iris.txt',
+                                     'data/model/list_fields.txt',
+                                     'data/model/tree_csv.txt'],
+            ['data/model/regression.json', 'data/model/rdistribution_iris.txt',
+                                           'data/model/rlist_fields.txt',
+                                           'data/model/rtree_csv.txt'],
+            ['data/model/w_iris.json', 'data/model/wdistribution_iris.txt',
+                                       'data/model/wlist_fields.txt',
+                                       'data/model/wtree_csv.txt'],
+            ['data/model/w_regression.json', 'data/model/wrdistribution_iris.txt',
+                                             'data/model/wrlist_fields.txt',
+                                             'data/model/wrtree_csv.txt']]
+        for example in examples:
+            print("\nTesting with:\n", example)
+            world.debug=True
+            model_create.i_read_model_file(self, example[0])
+            prediction_compare.i_create_a_local_model(self)
+            inspect_model.i_check_print_distribution(self, example[1])
+            inspect_model.i_list_fields(self, example[2])
+            inspect_model.i_create_tree_csv(self, example[3])
