@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
 #
-# Copyright 2015-2020 BigML
+# Copyright 2015-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -19,19 +18,18 @@
 """ Comparing remote and local predictions
 
 """
-from world import world, setup_module, teardown_module, show_doc
-from bigml.util import PY3
-import create_source_steps as source_create
-import create_dataset_steps as dataset_create
-import create_model_steps as model_create
-import create_ensemble_steps as ensemble_create
-import create_linear_steps as linear_create
-import create_association_steps as association_create
-import create_cluster_steps as cluster_create
-import create_anomaly_steps as anomaly_create
-import create_prediction_steps as prediction_create
-import compare_predictions_steps as prediction_compare
-import create_lda_steps as topic_create
+from .world import world, setup_module, teardown_module, show_doc
+from . import create_source_steps as source_create
+from . import create_dataset_steps as dataset_create
+from . import create_model_steps as model_create
+from . import create_ensemble_steps as ensemble_create
+from . import create_linear_steps as linear_create
+from . import create_association_steps as association_create
+from . import create_cluster_steps as cluster_create
+from . import create_anomaly_steps as anomaly_create
+from . import create_prediction_steps as prediction_create
+from . import compare_predictions_steps as prediction_compare
+from . import create_lda_steps as topic_create
 
 
 
@@ -41,13 +39,13 @@ class TestComparePrediction(object):
         """
             Debug information
         """
-        print "\n-------------------\nTests in: %s\n" % __name__
+        print("\n-------------------\nTests in: %s\n" % __name__)
 
     def teardown(self):
         """
             Debug information
         """
-        print "\nEnd of tests in: %s\n-------------------\n" % __name__
+        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
 
     def test_scenario1(self):
         """
@@ -81,11 +79,11 @@ class TestComparePrediction(object):
             ['data/spam.csv', '20', '20', '30', '{"fields": {"000001": {"optype": "text", "term_analysis": {"case_sensitive": true, "stem_words": true, "use_stopwords": false, "language": "en"}}}}', '{"Type": "", "Message": ""}', 'Cluster 6', '0.5'],
             ['data/diabetes.csv', '20', '20', '30', '{"fields": {}}', '{"pregnancies": 0, "plasma glucose": 118, "blood pressure": 84, "triceps skin thickness": 47, "insulin": 230, "bmi": 45.8, "diabetes pedigree": 0.551, "age": 31, "diabetes": "true"}', 'Cluster 3', '0.5033378686559257'],
             ['data/diabetes.csv', '20', '20', '30', '{"fields": {}}', '{"pregnancies": 0, "plasma glucose": 118, "blood pressure": 84, "triceps skin thickness": 47, "insulin": 230, "bmi": 45.8, "diabetes pedigree": 0.551, "age": 31, "diabetes": true}', 'Cluster 3', '0.5033378686559257'],
-            ['data/iris_sp_chars.csv', '20', '20', '30', '{"fields": {}}', '{"pétal.length":1, "pétal&width\u0000": 2, "sépal.length":1, "sépal&width": 2, "spécies": "Iris-setosa"}', 'Cluster 7', '0.8752380218327035'],
+            ['data/iris_sp_chars.csv', '20', '20', '30', '{"fields": {}}', '{"pétal.length":1, "pétal&width\\u0000": 2, "sépal.length":1, "sépal&width": 2, "spécies": "Iris-setosa"}', 'Cluster 7', '0.8752380218327035'],
             ['data/movies.csv', '20', '20', '30', '{"fields": {"000007": {"optype": "items", "item_analysis": {"separator": "$"}}}}', '{"gender": "Female", "age_range": "18-24", "genres": "Adventure$Action", "timestamp": 993906291, "occupation": "K-12 student", "zipcode": 59583, "rating": 3}', 'Cluster 3', '0.62852']]
         show_doc(self.test_scenario1, examples)
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             source_create.i_update_source_with(self, example[4])
@@ -122,7 +120,7 @@ class TestComparePrediction(object):
             ['data/iris.csv', '20', '20', '30', '{"default_numeric_value": "zero"}', '{"petal length": 1}', 'Cluster 4', '1.41215', '{"petal length": 1, "petal width": 0, "sepal length": 0, "sepal width": 0, "species": ""}']]
         show_doc(self.test_scenario2, examples)
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -143,7 +141,7 @@ class TestComparePrediction(object):
                 And I wait until the source is ready less than <time_1> secs
                 And I create a dataset
                 And I wait until the dataset is ready less than <time_2> secs
-                And I create an anomaly detector
+                And I create an anomaly detector with params "<params>"
                 And I wait until the anomaly detector is ready less than <time_3> secs
                 And I create a local anomaly detector
                 When I create an anomaly score for "<data_input>"
@@ -156,15 +154,18 @@ class TestComparePrediction(object):
 
         """
         examples = [
-            ['data/tiny_kdd.csv', '30', '30', '30', '{"000020": 255.0, "000004": 183.0, "000016": 4.0, "000024": 0.04, "000025": 0.01, "000026": 0.0, "000019": 0.25, "000017": 4.0, "000018": 0.25, "00001e": 0.0, "000005": 8654.0, "000009": "0", "000023": 0.01, "00001f": 123.0}', '0.69802']]
+            ['data/tiny_kdd.csv', '30', '30', '30', '{"000020": 255.0, "000004": 183.0, "000016": 4.0, "000024": 0.04, "000025": 0.01, "000026": 0.0, "000019": 0.25, "000017": 4.0, "000018": 0.25, "00001e": 0.0, "000005": 8654.0, "000009": "0", "000023": 0.01, "00001f": 123.0}', '0.69802', '{}'],
+            ['data/repeat_iris.csv', '30', '30', '30', '{"sepal width":3.5, "petal width": 0.2, "sepal length": 5.1, "petal length": 1.4, "species": "Iris-setosa"}', '0.50', '{"normalize_repeats": false}'],
+            ['data/repeat_iris.csv', '30', '30', '30', '{"sepal width":3.5, "petal width": 0.2, "sepal length": 5.1, "petal length": 1.4, "species": "Iris-setosa"}', '0.36692', '{"normalize_repeats": true}'],
+            ['data/repeat_iris.csv', '30', '30', '30', '{"sepal width":3.2, "petal width": 1.5, "sepal length": 6.4, "petal length": 4.5, "species": "Iris-versicolor"}', '0.76131', '{"normalize_repeats": true}']]
         show_doc(self.test_scenario3, examples)
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
             dataset_create.the_dataset_is_finished_in_less_than(self, example[2])
-            anomaly_create.i_create_an_anomaly(self)
+            anomaly_create.i_create_an_anomaly_with_params(self, example[6])
             anomaly_create.the_anomaly_is_finished_in_less_than(self, example[3])
             prediction_compare.i_create_a_local_anomaly(self)
             prediction_create.i_create_an_anomaly_score(self, example[4])
@@ -198,7 +199,7 @@ class TestComparePrediction(object):
             ['data/spam.csv', '30', '30', '30', '{"fields": {"000001": {"optype": "text", "term_analysis": {"case_sensitive": true, "stem_words": true, "use_stopwords": false, "language": "en"}}}}', '{"Type": "ham", "Message": "Go until jurong point, crazy.. Available only in bugis n great world la e buffet... Cine there got amore wat..."}', '[0.39188, 0.00643, 0.00264, 0.00643, 0.08112, 0.00264, 0.37352, 0.0115, 0.00707, 0.00327, 0.00264, 0.11086]']]
         show_doc(self.test_scenario4, examples)
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             source_create.i_update_source_with(self, example[4])
@@ -235,7 +236,7 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario5, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             source_create.i_update_source_with(self, example[4])
@@ -284,7 +285,7 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario6, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -324,14 +325,59 @@ class TestComparePrediction(object):
         examples = [
             ['data/iris.csv', '30', '30', '50', '{}', '000004', 'Iris-virginica', '0.33784', '{"boosting": {"iterations": 5}}', {}],
             ['data/iris.csv', '30', '30', '50', '{}', '000004', 'Iris-versicolor', '0.27261', '{"number_of_models": 5"}', {"operating_kind": "confidence"}],
-            ['data/grades.csv', '30', '30', '50', '{}', '000005', '70.505792', '30.7161', '{"number_of_models": 5}', {}],
+            ['data/grades.csv', '30', '30', '50', '{}', '000005', '70.505792', '30.7161', '{"number_of_models": 5}', {}]]
+
+        show_doc(self.test_scenario7, examples)
+
+        for example in examples:
+            print("\nTesting with:\n", example)
+            source_create.i_upload_a_file(self, example[0])
+            source_create.the_source_is_finished(self, example[1])
+            dataset_create.i_create_a_dataset(self)
+            dataset_create.the_dataset_is_finished_in_less_than(self, example[2])
+            ensemble_create.i_create_an_ensemble_with_params(self, example[8])
+            ensemble_create.the_ensemble_is_finished_in_less_than(self, example[3])
+            ensemble_create.create_local_ensemble(self)
+            prediction_create.i_create_an_ensemble_proportional_prediction(self, example[4], example[9])
+            prediction_create.the_prediction_is(self, example[5], example[6])
+            prediction_create.the_confidence_is(self, example[7])
+            prediction_create.create_local_ensemble_proportional_prediction_with_confidence(self, example[4], example[9])
+            prediction_compare.the_local_ensemble_prediction_is(self, example[6])
+            prediction_compare.the_local_prediction_confidence_is(self, example[7])
+
+
+
+    def test_scenario7b(self):
+        """
+            Scenario: Successfully comparing predictions for ensembles with proportional missing strategy:
+                Given I create a data source uploading a "<data>" file
+                And I wait until the source is ready less than <time_1> secs
+                And I create a dataset
+                And I wait until the dataset is ready less than <time_2> secs
+                And I create an esemble with "<params>"
+                And I wait until the ensemble is ready less than <time_3> secs
+                And I create a local ensemble
+                When I create a proportional missing strategy prediction for "<data_input>"
+                Then the prediction for "<objective>" is "<prediction>"
+                And the confidence for the prediction is "<confidence>"
+                And I create a proportional missing strategy local prediction for "<data_input>"
+                Then the local prediction is "<prediction>"
+                And the local prediction's confidence is "<confidence>"
+
+                Examples:
+                | data               | time_1  | time_2 | time_3 | data_input           | objective | prediction     | confidence | params
+            ['data/iris.csv', '10', '10', '50', '{}', '000004', 'Iris-virginica', '0.33784', '{"boosting": {"iterations": 5}}'],
+
+
+        """
+        examples = [
             ['data/grades.csv', '30', '30', '50', '{"Midterm": 20}', '000005', '54.82214', '25.89672', '{"number_of_models": 5}', {"operating_kind": "confidence"}],
             ['data/grades.csv', '30', '30', '50', '{"Midterm": 20}', '000005', '45.4573', '29.58403', '{"number_of_models": 5}', {}],
             ['data/grades.csv', '30', '30', '50', '{"Midterm": 20, "Tutorial": 90, "TakeHome": 100}', '000005', '42.814', '31.51804', '{"number_of_models": 5}', {}]]
         show_doc(self.test_scenario7, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -362,7 +408,7 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario6, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             ensemble_create.create_local_ensemble_predictor(self, example[0])
             prediction_compare.i_create_a_local_ensemble_prediction(self, example[1])
             prediction_compare.the_local_ensemble_prediction_is(self, example[2])
@@ -396,7 +442,7 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario7, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -444,8 +490,8 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario10, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
-            tag = "%s_%s" % (example[5], PY3)
+            print("\nTesting with:\n", example)
+            tag = example[5]
             tag_args = '{"tags":["%s"]}' % tag
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
@@ -501,8 +547,8 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario11, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
-            tag = "%s_%s" % (example[5], PY3)
+            print("\nTesting with:\n", example)
+            tag = example[5]
             tag_args = '{"tags":["%s"]}' % tag
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
@@ -549,15 +595,15 @@ class TestComparePrediction(object):
                 | data             | time_1  | time_2 | time_3 | params| tag | data_input                             | objective | prediction  | params
 
         """
-        tag = "my_fusion_tag_12_%s" % PY3
-        tag_reg = "my_fusion_tag_12_reg_%s" % PY3
+        tag = "my_fusion_tag_12"
+        tag_reg = "my_fusion_tag_12_reg"
         examples = [
             ['data/iris_unbalanced.csv', '30', '30', '120', '120', '{"tags":["%s"], "sample_rate": 0.8, "seed": "bigml"}' % tag, tag, '{"petal width": 4}', '000004', 'Iris-virginica'],
             ['data/grades.csv', '30', '30', '120', '120', '{"tags":["%s"], "sample_rate": 0.8, "seed": "bigml"}' % tag_reg, tag_reg, '{"Midterm": 20}', '000005', 44.37625]]
         show_doc(self.test_scenario12, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -609,8 +655,8 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario13, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
-            tag = "%s_%s" % (example[5], PY3)
+            print("\nTesting with:\n", example)
+            tag = example[5]
             tag_args = '{"tags":["%s"]}' % tag
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
