@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
 #
-# Copyright 2017-2020 BigML
+# Copyright 2017-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -19,14 +18,14 @@
 """ Comparing remote and local predictions
 
 """
-from world import world, setup_module, teardown_module, show_doc
-import create_source_steps as source_create
-import create_dataset_steps as dataset_create
-import create_model_steps as model_create
-import create_ensemble_steps as ensemble_create
-import create_linear_steps as linear_create
-import create_prediction_steps as prediction_create
-import compare_predictions_steps as prediction_compare
+from .world import world, setup_module, teardown_module, show_doc
+from . import create_source_steps as source_create
+from . import create_dataset_steps as dataset_create
+from . import create_model_steps as model_create
+from . import create_ensemble_steps as ensemble_create
+from . import create_linear_steps as linear_create
+from . import create_prediction_steps as prediction_create
+from . import compare_predictions_steps as prediction_compare
 
 
 class TestComparePrediction(object):
@@ -35,13 +34,13 @@ class TestComparePrediction(object):
         """
             Debug information
         """
-        print "\n-------------------\nTests in: %s\n" % __name__
+        print("\n-------------------\nTests in: %s\n" % __name__)
 
     def teardown(self):
         """
             Debug information
         """
-        print "\nEnd of tests in: %s\n-------------------\n" % __name__
+        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
 
     def test_scenario1(self):
         """
@@ -61,18 +60,19 @@ class TestComparePrediction(object):
                 Examples:
                 | data             | time_1  | time_2 | time_3 | data_input                             | objective | prediction  | params,
 
-
+,
+            ['data/spam.csv', '30', '50', '60', '{}', '000000', 'ham', '{}']
         """
         examples = [
             ['data/iris.csv', '30', '50', '60', '{"petal width": 4}', '000004', 'Iris-virginica', '{}'],
-            ['data/iris.csv', '30', '50', '60', '{"sepal length": 4.1, "sepal width": 2.4}', '000004', 'Iris-setosa', '{}'],
-            ['data/iris_missing2.csv', '30', '50', '60', '{}', '000004', 'Iris-setosa', '{}'],
-            ['data/grades.csv', '30', '50', '60', '{}', '000005', 42.15473, '{}'],
+            ['data/iris.csv', '30', '50', '60', '{"sepal length": 4.1, "sepal width": 2.4}', '000004', 'Iris-versicolor', '{}'],
+            ['data/iris_missing2.csv', '30', '50', '60', '{}', '000004', 'Iris-versicolor', '{}'],
+            ['data/grades.csv', '30', '50', '60', '{}', '000005', 55.65609, '{}'],
             ['data/spam.csv', '30', '50', '60', '{}', '000000', 'ham', '{}']]
         show_doc(self.test_scenario1, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -114,7 +114,7 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario2, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -149,11 +149,11 @@ class TestComparePrediction(object):
 
         """
         examples = [
-            ['data/iris.csv', '10', '50', '60', '{"petal width": 4}', '000004', 'Iris-versicolor', '{}', {"kind": "probability", "threshold": 1, "positive_class": "Iris-virginica"}]]
+            ['data/iris.csv', '10', '50', '60', '{"petal width": 4}', '000004', 'Iris-setosa', '{}', {"kind": "probability", "threshold": 1, "positive_class": "Iris-virginica"}]]
         show_doc(self.test_scenario3, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -195,7 +195,7 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario4, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -237,7 +237,7 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario5, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -277,7 +277,7 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario6, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -321,7 +321,7 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario7, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -360,7 +360,7 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario8, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -400,7 +400,7 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario9, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -440,7 +440,7 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario10, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -498,7 +498,7 @@ class TestComparePrediction(object):
         show_doc(self.test_scenario11, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
@@ -568,33 +568,33 @@ class TestComparePrediction(object):
         examples = [
             ['data/dates2.csv', '20', '45', '60',
              '{"time-1": "1910-05-08T19:10:23.106", "cat-0":"cat2"}',
-             '000002', 0.04082],
+             '000002', -0.02616],
             ['data/dates2.csv', '20', '45', '60',
              '{"time-1": "2011-04-01T00:16:45.747", "cat-0":"cat2"}',
-             '000002', 0.02919],
+             '000002', 0.13352],
             ['data/dates2.csv', '20', '45', '60',
              '{"time-1": "1969-W29-1T17:36:39Z", "cat-0":"cat1"}',
-             '000002', 0.0199],
+             '000002', 0.10071],
             ['data/dates2.csv', '20', '45', '60',
              '{"time-1": "1920-06-45T20:21:20.320", "cat-0":"cat1"}',
-             '000002', 0.0199],
+             '000002', 0.10071],
             ['data/dates2.csv', '20', '45', '60',
              '{"time-1": "2001-01-05T23:04:04.693", "cat-0":"cat2"}',
-             '000002', 0.28517],
+             '000002', 0.15235],
             ['data/dates2.csv', '20', '45', '60',
              '{"time-1": "1950-11-06T05:34:05.602", "cat-0":"cat1"}',
-             '000002', -0.05673],
+             '000002', -0.07686],
             ['data/dates2.csv', '20', '45', '60',
              '{"time-1": "1932-01-30T19:24:11.440",  "cat-0":"cat2"}',
-             '000002', 0.16183],
+             '000002', 0.0017],
             ['data/dates2.csv', '20', '45', '60',
              '{"time-1": "Mon Jul 14 17:36 +0000 1969", "cat-0":"cat1"}',
-             '000002', 0.0199]
+             '000002', 0.10071]
 ]
         show_doc(self.test_scenario12, examples)
 
         for example in examples:
-            print "\nTesting with:\n", example
+            print("\nTesting with:\n", example)
             source_create.i_upload_a_file(self, example[0])
             source_create.the_source_is_finished(self, example[1])
             dataset_create.i_create_a_dataset(self)
