@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-#!/usr/bin/env python
 #
-# Copyright 2012-2020 BigML
+# Copyright 2012-2021 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -29,14 +28,13 @@ import math
 import random
 import ast
 import datetime
-from urlparse import urlparse
 
-import unidecode
+from urllib.parse import urlparse
 
+import msgpack
 
 import bigml.constants as c
 
-PY3 = sys.version > '3'
 DEFAULT_LOCALE = 'en_US.UTF-8'
 WINDOWS_DEFAULT_LOCALE = 'English'
 LOCALE_SYNONYMS = {
@@ -87,10 +85,10 @@ TYPE_MAP = {
 }
 
 PYTHON_TYPE_MAP = {
-    "categorical": [unicode, str],
+    "categorical": [str, str],
     "numeric": [int, float],
-    "text": [unicode, str],
-    "items": [unicode, str]
+    "text": [str, str],
+    "items": [str, str]
 }
 
 PREDICTIONS_FILE_SUFFIX = '_predictions.csv'
@@ -106,7 +104,7 @@ NUMERIC = "numeric"
 DFT_STORAGE = "./storage"
 DFT_STORAGE_FILE = os.path.join(DFT_STORAGE, "BigML_%s.json")
 
-DECIMAL_DIGITS= 5
+DECIMAL_DIGITS = 5
 
 
 def python_map_type(value):
@@ -115,8 +113,7 @@ def python_map_type(value):
     """
     if value in PYTHON_TYPE_MAP:
         return PYTHON_TYPE_MAP[value]
-    else:
-        return [unicode, str]
+    return [str, str]
 
 
 def invert_dictionary(dictionary, field='name'):
@@ -126,25 +123,7 @@ def invert_dictionary(dictionary, field='name'):
     It does not check whether new keys are duplicated though.
 
     """
-    return dict([[value[field], key]
-                 for key, value in dictionary.items()])
-
-
-def slugify(name, reserved_keywords=None, prefix=''):
-    """Translates a field name into a variable name.
-
-    """
-    name = unidecode.unidecode(name).lower()
-    name = re.sub(r'\W+', '_', name)
-    try:
-        if name[0].isdigit():
-            name = "field_" + name
-    except IndexError:
-        name = "unnamed_field"
-    if reserved_keywords:
-        if name in reserved_keywords:
-            name = prefix + name
-    return name
+    return {value[field]: key for key, value in dictionary.items()}
 
 
 def localize(number):
@@ -158,18 +137,16 @@ def is_url(value):
     """Returns True if value is a valid URL.
 
     """
-    url = isinstance(value, basestring) and urlparse(value)
+    url = isinstance(value, str) and urlparse(value)
     return url and url.scheme and url.netloc and url.path
 
 
-def split(children):
-    """Returns the field that is used by the node to make a decision.
+def is_in_progress(resource):
+    """Returns True if the resource has no error and has not finished yet
 
     """
-    field = set([child.predicate.field for child in children])
-
-    if len(field) == 1:
-        return field.pop()
+    return resource.get("error") is None \
+        and get_status(resource).get("code") != c.FINISHED
 
 
 def markdown_cleanup(text):
@@ -208,36 +185,14 @@ def prefix_as_comment(comment_prefix, text):
     return text.replace('\n', '\n' + comment_prefix)
 
 
-def sort_fields(fields):
-    """Sort fields by their column_number but put children after parents.
+def utf8(bytes_str):
+    """Returns utf-8 string for bytes or string objects
 
     """
-    fathers = [(key, val) for key, val in
-               sorted(fields.items(), key=lambda k: k[1]['column_number'])
-               if 'auto_generated' not in val]
-    children = [(key, val) for key, val in
-                sorted(fields.items(), key=lambda k: k[1]['column_number'])
-                if 'auto_generated' in val]
-    children.reverse()
-    fathers_keys = [father[0] for father in fathers]
-    for child in children:
-        try:
-            index = fathers_keys.index(child[1]['parent_ids'][0])
-        except ValueError:
-            index = -1
-
-        if index >= 0:
-            fathers.insert(index + 1, child)
-        else:
-            fathers.append(child)
-    return fathers
-
-
-def utf8(text):
-    """Returns text in utf-8 encoding
-
-    """
-    return text.encode("utf-8")
+    try:
+        return str(bytes_str, 'utf-8')
+    except TypeError:
+        return bytes_str
 
 
 def map_type(value):
@@ -246,8 +201,7 @@ def map_type(value):
     """
     if value in TYPE_MAP:
         return TYPE_MAP[value]
-    else:
-        return str
+    return str
 
 
 def locale_synonyms(main_locale, locale_alias):
@@ -258,15 +212,14 @@ def locale_synonyms(main_locale, locale_alias):
     if language_code not in LOCALE_SYNONYMS:
         return False
     alternatives = LOCALE_SYNONYMS[language_code]
-    if isinstance(alternatives[0], basestring):
+    if isinstance(alternatives[0], str):
         return main_locale in alternatives and locale_alias in alternatives
-    else:
-        result = False
-        for subgroup in alternatives:
-            if main_locale in subgroup:
-                result = locale_alias in subgroup
-                break
-        return result
+    result = False
+    for subgroup in alternatives:
+        if main_locale in subgroup:
+            result = locale_alias in subgroup
+            break
+    return result
 
 
 def bigml_locale(locale_alias):
@@ -279,16 +232,15 @@ def bigml_locale(locale_alias):
     if language_code not in LOCALE_SYNONYMS:
         return None
     alternatives = LOCALE_SYNONYMS[language_code]
-    if isinstance(alternatives[0], basestring):
+    if isinstance(alternatives[0], str):
         return (alternatives[0] if locale_alias in alternatives
                 else None)
-    else:
-        result = None
-        for subgroup in alternatives:
-            if locale_alias in subgroup:
-                result = subgroup[0]
-                break
-        return result
+    result = None
+    for subgroup in alternatives:
+        if locale_alias in subgroup:
+            result = subgroup[0]
+            break
+    return result
 
 
 def find_locale(data_locale=DEFAULT_LOCALE, verbose=False):
@@ -336,9 +288,9 @@ def find_locale(data_locale=DEFAULT_LOCALE, verbose=False):
         new_locale = locale.setlocale(locale.LC_NUMERIC, '')
 
     if verbose and not locale_synonyms(data_locale, new_locale):
-        print ("WARNING: Unable to find %s locale, using %s instead. This "
+        print(("WARNING: Unable to find %s locale, using %s instead. This "
                "might alter numeric fields values.\n") % (data_locale,
-                                                          new_locale)
+                                                          new_locale))
 
 
 def get_predictions_file_name(model, path):
@@ -381,10 +333,6 @@ def console_log(message, out=sys.stdout, length=PROGRESS_BAR_WIDTH,
     if reset:
         clear_console_line(out=out, length=length)
         reset_console_line(out=out, length=length)
-    if (out == sys.stdout and sys.platform == "win32" and sys.stdout.isatty()
-            and not PY3):
-        os_encoding = sys.stdout.encoding
-        message = message.decode('utf8').encode(os_encoding, 'replace')
     out.write(message)
     if reset:
         reset_console_line(out=out, length=length)
@@ -402,8 +350,8 @@ def strip_affixes(value, field):
     """Strips prefixes and suffixes if present
 
     """
-    if not isinstance(value, unicode):
-        value = unicode(value, "utf-8")
+    if not isinstance(value, str):
+        value = str(value, "utf-8")
     if 'prefix' in field and value.startswith(field['prefix']):
         value = value[len(field['prefix']):]
     if 'suffix' in field and value.endswith(field['suffix']):
@@ -415,7 +363,7 @@ def cast(input_data, fields):
     """Checks expected type in input data values, strips affixes and casts
 
     """
-    for (key, value) in input_data.items():
+    for (key, value) in list(input_data.items()):
         # strings given as booleans
         if isinstance(value, bool) and \
                 fields[key]['optype'] == 'categorical' and \
@@ -432,15 +380,15 @@ def cast(input_data, fields):
                 # converting boolean to the corresponding string
                 input_data.update({key: booleans[str(value)]})
             except ValueError:
-                raise ValueError(u"Mismatch input data type in field "
-                                 u"\"%s\" for value %s. String expected" %
+                raise ValueError("Mismatch input data type in field "
+                                 "\"%s\" for value %s. String expected" %
                                  (fields[key]['name'], value))
         # numerics given as strings
         elif (
                 (fields[key]['optype'] == NUMERIC and
-                 isinstance(value, basestring)) or
+                 isinstance(value, str)) or
                 (fields[key]['optype'] != NUMERIC and
-                 not isinstance(value, basestring))):
+                 not isinstance(value, str))):
             try:
                 if fields[key]['optype'] == NUMERIC:
                     value = strip_affixes(value, fields[key])
@@ -448,14 +396,14 @@ def cast(input_data, fields):
                                    map_type(fields[key]
                                             ['optype'])(value)})
             except ValueError:
-                raise ValueError(u"Mismatch input data type in field "
-                                 u"\"%s\" for value %s." %
+                raise ValueError("Mismatch input data type in field "
+                                 "\"%s\" for value %s." %
                                  (fields[key]['name'],
                                   value))
         elif (fields[key]['optype'] == NUMERIC and
               isinstance(value, bool)):
-            raise ValueError(u"Mismatch input data type in field "
-                             u"\"%s\" for value %s. Numeric expected." %
+            raise ValueError("Mismatch input data type in field "
+                             "\"%s\" for value %s. Numeric expected." %
                              (fields[key]['name'], value))
         if fields[key]['optype'] == NUMERIC and isinstance(value, float):
             input_data.update({key: round(value, DECIMAL_DIGITS)})
@@ -467,7 +415,7 @@ def check_dir(path):
     """
     if os.path.exists(path):
         if not os.path.isdir(path):
-            raise ValueError(u"The given path is not a directory")
+            raise ValueError("The given path is not a directory")
     elif len(path) > 0:
         os.makedirs(path)
     return path
@@ -540,11 +488,12 @@ def is_status_final(resource):
     """Try whether a resource is in a final state
 
     """
+    status = {}
     try:
         status = get_status(resource)
     except ValueError:
-        status['code'] = None
-    return status['code'] in [c.FINISHED, c.FAULTY]
+        pass
+    return status.get('code') in [c.FINISHED, c.FAULTY]
 
 
 def save_json(resource, path):
@@ -555,9 +504,9 @@ def save_json(resource, path):
         resource_json = json.dumps(resource)
         return save(resource_json, path)
     except ValueError:
-        print "The resource has an invalid JSON format"
+        print("The resource has an invalid JSON format")
     except IOError:
-        print "Failed writing resource to %s" % path
+        print("Failed writing resource to %s" % path)
 
 
 def save(content, path):
@@ -597,7 +546,7 @@ def check_no_missing_numerics(input_data, fields, weight_field=None):
     """Checks whether some numeric fields are missing in the input data
 
     """
-    for field_id, field in fields.items():
+    for field_id, field in list(fields.items()):
         if (field['optype'] == NUMERIC and (weight_field is None or \
                 field_id != weight_field) and \
                 not field_id in input_data):
@@ -605,13 +554,14 @@ def check_no_missing_numerics(input_data, fields, weight_field=None):
                              " data must contain values for all numeric"
                              " fields to get a prediction.")
 
+
 def check_no_training_missings(input_data, fields, weight_field=None,
                                objective_id=None):
     """Checks whether some input fields are missing in the input data
     while not training data has no missings in that field
 
     """
-    for field_id, field in fields.items():
+    for field_id, field in list(fields.items()):
         if (field["optype"] != "datetime" and \
                 field_id not in input_data and \
                 field['summary']['missing_count'] == 0 and \
@@ -635,3 +585,39 @@ def flatten(inner_array):
             new_array.append(element)
 
     return new_array
+
+
+def use_cache(cache_get):
+    """Checks whether the user has provided a cache get function to retrieve
+       local models.
+
+    """
+    return cache_get is not None and hasattr(cache_get, '__call__')
+
+
+def dump(local_attrs, output=None, cache_set=None):
+    """Uses msgpack to serialize the local resource object
+    If cache_set is filled with a cache set method, the method is called
+
+    """
+    if use_cache(cache_set):
+        dump_string = msgpack.dumps(local_attrs)
+        cache_set(local_attrs["resource_id"], dump_string)
+    else:
+        msgpack.pack(local_attrs, output)
+
+
+def dumps(local_attrs):
+    """Uses msgpack to serialize the anomaly object to a string
+
+    """
+
+    return msgpack.dumps(local_attrs)
+
+
+def load(resource_id, cache_get):
+    """Uses msgpack to load the resource stored by ID
+
+    """
+
+    return msgpack.loads(cache_get(resource_id))
